#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
# Name:         test_tonality.py
# Purpose:      Pymuco Unit Tests
#
# Authors:      German Margon
#
# Copyright:    Copyright © 2022-2023 German Margon. All rights reserved.
# License:      BSD 3-Clause License, see LICENSE
# ------------------------------------------------------------------------------
import unittest

from pymuco import Tonality, ScientificPitchNotation


class TestTonality(unittest.TestCase):
    """
    A class containing unit tests for the Tonality class.

    Attributes
    ----------
    None

    Methods
    -------
    test_tonality_creation()
        Test method for the Tonality class constructor.

    Examples
    --------
    Create a TestTonality object and run its test methods:

    >>> test = TestTonality()
    >>> test.test_tonality_creation()
    """

    def test_tonality_creation(self):
        """
        Test method for the Tonality class constructor.

        Parameters
        ----------
        self : TestTonality instance
            The current test instance.

        Returns
        -------
        None

        Notes
        -----
        This test method creates a Tonality object with a root note and
        tonality type, and checks that the object is an instance of the
        Tonality class and that its attributes match the expected values.

        Example
        -------
        >>> test = TestTonality()
        >>> test.test_tonality_creation()
        """
        root_note = ScientificPitchNotation("C", 4)
        tonality_type = "major"
        tonality = Tonality(root_note, tonality_type)
        self.assertIsInstance(tonality, Tonality)
        self.assertEqual(tonality._root_note, root_note)
        self.assertEqual(tonality._tonality_type, tonality_type)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
# Name:         test_enharmonic_mapping.py
# Purpose:      Pymuco Unit Tests
#
# Authors:      German Margon
#
# Copyright:    Copyright © 2022-2023 German Margon. All rights reserved.
# License:      BSD 3-Clause License, see LICENSE
# ------------------------------------------------------------------------------
import unittest

from pymuco import EnharmonicMapping, ScientificPitchNotation


class TestEnharmonicMapping(unittest.TestCase):
    """
    Test case for EnharmonicMapping class.

    This class contains test methods to ensure that EnharmonicMapping class
    behaves as expected by checking that calling the `enharmonic_mapping()`
    method returns the expected dictionary.

    Attributes
    ----------
    _ENHARMONIC_MAPPING : dict
        A dictionary mapping notes to their enharmonic equivalents.
    enharmonic_mapping : EnharmonicMapping
        An instance of EnharmonicMapping to use in the test methods.

    Examples
    --------
    >>> class TestEnharmonicMapping(unittest.TestCase):
    ...     def setUp(self):
    ...         self.enharmonic_mapping = EnharmonicMapping()
    ...
    ...     def test_enharmonic_mapping(self):
    ...         for key, value in self._ENHARMONIC_MAPPING.items():
    ...             self.assertEqual(
    ...                 self.enharmonic_mapping.enharmonic_mapping(),
    ...                 self._ENHARMONIC_MAPPING,
    ...             )
    ...
    >>> test = TestEnharmonicMapping()
    >>> test.test_enharmonic_mapping()
    """

    _ENHARMONIC_MAPPING = {
        "Ax": "B",
        "A#": "Bb",
        "A": "A",
        "Ab": "G#",
        "A𝄫": "G",
        "Bx": "C#",
        "B#": "C",
        "B": "B",
        "Bb": "A#",
        "B𝄫": "A",
        "Cx": "D",
        "C#": "Db",
        "C": "C",
        "Cb": "B",
        "C𝄫": "Bb",
        "Dx": "E",
        "D#": "Eb",
        "D": "D",
        "Db": "C#",
        "D𝄫": "C",
        "Ex": "F#",
        "E#": "F",
        "E": "E",
        "Eb": "D#",
        "E𝄫": "D",
        "Fx": "G",
        "F#": "Gb",
        "F": "F",
        "Fb": "E",
        "F𝄫": "Eb",
        "Gx": "A",
        "G#": "Ab",
        "G": "G",
        "Gb": "F#",
        "G𝄫": "F",
    }

    def setUp(self):
        """
        Set up the test fixture for EnharmonicMapping class.

        This method initializes an instance of EnharmonicMapping class and
        assigns it to the instance variable `enharmonic_mapping`.

        Parameters
        ----------
        self: object
            The instance of the test class.

        Returns
        -------
        None

        Examples
        --------
        >>> class TestEnharmonicMapping(unittest.TestCase):
        ...     def setUp(self):
        ...         self.enharmonic_mapping = EnharmonicMapping()
        ...
        >>> test = TestEnharmonicMapping()
        >>> test.setUp()
        """
        self.enharmonic_mapping = EnharmonicMapping()

    def test_enharmonic_mapping(self):
        """
        Test that EnharmonicMapping class correctly maps enharmonic
        equivalents.

        This method iterates over each key-value pair in the
        _ENHARMONIC_MAPPING dictionary and asserts that calling the
        `enharmonic_mapping()` method of a new EnharmonicMapping instance
        returns the expected dictionary. This ensures that all the expected
        enharmonic mappings are present in the implementation.

        Parameters
        ----------
        self: object
            The instance of the test class.

        Returns
        -------
        None

        Examples
        --------
        >>> class TestEnharmonicMapping(unittest.TestCase):
        ...     def test_enharmonic_mapping(self):
        ...         for key, value in self._ENHARMONIC_MAPPING.items():
        ...             self.assertEqual(
        ...                 EnharmonicMapping().enharmonic_mapping(),
        ...                 self._ENHARMONIC_MAPPING,
        ...             )
        ...
        >>> test = TestEnharmonicMapping()
        >>> test.test_enharmonic_mapping()
        """
        for key, value in self._ENHARMONIC_MAPPING.items():
            self.assertEqual(
                EnharmonicMapping().enharmonic_mapping(),
                self._ENHARMONIC_MAPPING,
            )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
# Name:         test_scale.py
# Purpose:      Pymuco Unit Tests
#
# Authors:      German Margon
#
# Copyright:    Copyright © 2022-2023 German Margon. All rights reserved.
# License:      BSD 3-Clause License, see LICENSE
# ------------------------------------------------------------------------------
import unittest

from pymuco import Scale, ScaleData, ScientificPitchNotation


class TestScaleData(unittest.TestCase):
    """
    This is a test case for the `ScaleData` class.

    Test methods:
    -------------
    test_major_scale_sequence:
        Tests the `major_scale_sequence` method of the `ScaleData` class.
    test_minor_scale_sequence:
        Tests the `minor_scale_sequence` method of the `ScaleData` class.

    See Also:
    ---------
    `ScaleData.major_scale_sequence`
    `ScaleData.minor_scale_sequence`
    """

    _SCALE_SEQUENCE = {
        "Major": [0, 2, 4, 5, 7, 9, 11, 12],
        "Minor": [0, 2, 3, 5, 7, 8, 10, 12],
    }

    def test_major_scale_sequence(self):
        """
        Unit test for the major_scale_sequence method of the ScaleData class.

        This test checks if the major_scale_sequence method of the ScaleData
        class returns the expected major scale sequence.

        Parameters
        ----------
        self : TestCase
            An instance of the unittest.TestCase class.

        Returns
        -------
        None

        Raises
        ------
        AssertionError
            If the returned value from the major_scale_sequence method does not
            match the expected major scale sequence.

        See Also
        --------
        ScaleData.major_scale_sequence : Method being tested.

        Notes
        -----
        This test compares the output of the major_scale_sequence method of the
        ScaleData class with the expected major scale sequence using the
        assertEqual method. This ensures that the major_scale_sequence method
        correctly returns the expected sequence.

        """
        self.assertEqual(
            ScaleData.SCALE_SEQUENCE.value["Major"],
            self._SCALE_SEQUENCE["Major"],
        )

    def test_minor_scale_sequence(self):
        """
        Test the `minor_scale_sequence` method of the `ScaleData` class.

        Tests if the `minor_scale_sequence` method of the `ScaleData` class
        returns the expected sequence of intervals for the Minor scale.

        Parameters:
        -----------
        self : object
            An instance of the `ScaleData` test class.

        Returns:
        --------
        None.

        Test method:
        test_minor_scale_sequence(self)

        Tested class:
        `ScaleData`

        Tested method:
        `minor_scale_sequence`

        Test procedure:
        ---------------
        1. Call the `minor_scale_sequence` method of the `ScaleData` class.
        2. Compare the returned value with the expected sequence of intervals
        for the Minor scale using the `assertEqual` method.

        Expected behavior:
        ------------------
        The `minor_scale_sequence` method of the `ScaleData` class should
        return the expected sequence of intervals for the Minor scale.

        See Also:
        ---------
        `ScaleData.SCALE_SEQUENCE`
        """
        self.assertEqual(
            ScaleData.SCALE_SEQUENCE.value["Minor"],
            self._SCALE_SEQUENCE["Minor"],
        )


class TestScale(unittest.TestCase):
    """
    The TestScale class is a subclass of unittest.TestCase, containing two test methods: test_get_major_scale and test_get_minor_scale.

    The class has two class-level dictionaries, _MAJOR_SCALE and _MINOR_SCALE, that define the notes of major and minor scales respectively.
    The test_get_major_scale method tests if the get_major_scale method of the Scale class returns the expected major scale for each note
    in the _MAJOR_SCALE dictionary. The test_get_minor_scale method tests if the get_scale method of the Scale class returns the expected scale
    for each note in the _MINOR_SCALE dictionary.
    """

    _MAJOR_SCALE = {
        "C": ["C", "D", "E", "F", "G", "A", "B", "C"],
        "D": ["D", "E", "F#", "G", "A", "B", "C#", "D"],
        "E": ["E", "F#", "G#", "A", "B", "C#", "D#", "E"],
        "F": ["F", "G", "A", "Bb", "C", "D", "E", "F"],
        "G": ["G", "A", "B", "C", "D", "E", "F#", "G"],
        "A": ["A", "B", "C#", "D", "E", "F#", "G#", "A"],
        "B": ["B", "C#", "D#", "E", "F#", "G#", "A#", "B"],
        "Bb": ["Bb", "C", "D", "Eb", "F", "G", "A", "Bb"],
        "Eb": ["Eb", "F", "G", "Ab", "Bb", "C", "D", "Eb"],
        "Ab": ["Ab", "Bb", "C", "Db", "Eb", "F", "G", "Ab"],
        "Db": ["Db", "Eb", "F", "Gb", "Ab", "Bb", "C", "Db"],
        "Gb": ["Gb", "Ab", "Bb", "B", "Db", "Eb", "F", "Gb"],
        "F#": ["F#", "G#", "A#", "B", "C#", "D#", "E#", "F#"],
    }

    _MINOR_SCALE = {
        "A": ["A", "B", "C", "D", "E", "F", "G", "A"],
        "E": ["E", "F#", "G", "A", "B", "C", "D", "E"],
        "D": ["D", "E", "F", "G", "A", "Bb", "C", "D"],
        "F#": ["F#", "G#", "A", "B", "C#", "D", "E", "F#"],
        "C#": ["C#", "D#", "E", "F#", "G#", "A", "B", "C#"],
        "G#": ["G#", "A#", "B", "C#", "D#", "E", "F#", "G#"],
        "D#": ["D#", "E#", "F#", "G#", "A#", "B", "C#", "D#"],
        "Eb": ["Eb", "F", "Gb", "Ab", "Bb", "B", "Db", "Eb"],
        "Bb": ["Bb", "C", "Db", "Eb", "F", "Gb", "Ab", "Bb"],
        "F": ["F", "G", "Ab", "Bb", "C", "Db", "Eb", "F"],
        "C": ["C", "D", "Eb", "F", "G", "Ab", "Bb", "C"],
        "G": ["G", "A", "Bb", "C", "D", "Eb", "F", "G"],
        "D": ["D", "E", "F", "G", "A", "Bb", "C", "D"],
    }

    def test_get_major_scale(self):
        """
        Unit test for the get_major_scale method of the Scale class.

        This test checks if the get_major_scale method of the Scale class returns the expected major scale for each note in the _MAJOR_SCALE dictionary. The method is called for each key-value pair in the _MAJOR_SCALE dictionary and the returned value is compared with the expected major scale using the assertListEqual method.

        Test method:
        test_get_major_scale(self)

        Tested class:
        Scale

        Tested method:
        get_major_scale

        Test procedure:

        Iterate through each key-value pair in the _MAJOR_SCALE dictionary.
        Create a ScientificPitchNotation object with the current note and octave 4.
        Call the get_major_scale method of the Scale class with the created ScientificPitchNotation object and the mode 'M'.
        Compare the returned scale with the expected major scale using the assertListEqual method.
        Expected behavior:
        The get_major_scale method of the Scale class should return the expected major scale for each note in the _MAJOR_SCALE dictionary.
        """
        for note, expected_scale in self._MAJOR_SCALE.items():
            with self.subTest(note=note, expected_scale=expected_scale):
                spn = ScientificPitchNotation(note, 4)
                scale = Scale(spn, "M").get_scale()
                self.assertListEqual(scale, expected_scale)

    def test_get_minor_scale(self):
        """
        Unit test for the get_scale method of the Scale class for minor scales.

        This test checks if the get_scale method of the Scale class returns the expected scale for each note in the _MINOR_SCALE dictionary. The method is called for each key-value pair in the _MINOR_SCALE dictionary and the returned value is compared with the expected scale using the assertListEqual method.

        Test method:
        test_get_minor_scale(self)

        Tested class:
        Scale

        Tested method:
        get_scale

        Test procedure:

        Iterate through each key-value pair in the _MINOR_SCALE dictionary.
        Create a ScientificPitchNotation object with the note and octave from the key.
        Call the get_scale method of the Scale class with the note and "m" as the scale type.
        Compare the returned value with the expected scale from the value of the key in the _MINOR_SCALE dictionary using the assertListEqual method.
        Expected behavior:
        The get_scale method of the Scale class should return the expected scale for each note in the _MINOR_SCALE dictionary.
        """
        for note, expected_scale in self._MINOR_SCALE.items():
            with self.subTest(note=note, expected_scale=expected_scale):
                spn = ScientificPitchNotation(note, 4)
                scale = Scale(spn, "m").get_scale()
                self.assertListEqual(scale, expected_scale)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
# Name:         test_enharmonic.py
# Purpose:      Pymuco Unit Tests
#
# Authors:      German Margon
#
# Copyright:    Copyright © 2022-2023 German Margon. All rights reserved.
# License:      BSD 3-Clause License, see LICENSE
# ------------------------------------------------------------------------------
import unittest

from pymuco import (
    Enharmonic,
    DEFAULT_OCTAVE,
    ScientificPitchNotation
)


class TestEnharmonic(unittest.TestCase):
    """
    Unittest class for the Enharmonic class.

    This class tests the functionality of the Enharmonic class, which provides
    methods for obtaining the enharmonic equivalent of a given note in Western
    music notation.

    The class contains the following unit tests:

    test_get_enharmonic:
    Tests if the get_enharmonic method returns the correct enharmonic note
    value for each note in the _ENHARMONIC_MAPPING dictionary.

    test_is_enharmonic:
    Tests if the is_enharmonic method returns True for each note in the
    _ENHARMONIC_MAPPING dictionary.

    test_get_all_enharmonics:
    Tests if the get_all_enharmonics method returns the expected list of
    enharmonic notes for a given note.

    The _ENHARMONIC_MAPPING dictionary maps each note name to its enharmonic
    equivalent, accounting for double sharps and flats.

    This class inherits from unittest.TestCase, which provides the testing
    functionality for each test case. The setUp method is not required since
    the tests do not require any specific setup.
    """

    _ENHARMONIC_MAPPING = {
        "Ax": "B",
        "A#": "Bb",
        "A": "A",
        "Ab": "G#",
        "A𝄫": "G",
        "Bx": "C#",
        "B#": "C",
        "B": "B",
        "Bb": "A#",
        "B𝄫": "A",
        "Cx": "D",
        "C#": "Db",
        "C": "C",
        "Cb": "B",
        "C𝄫": "Bb",
        "Dx": "E",
        "D#": "Eb",
        "D": "D",
        "Db": "C#",
        "D𝄫": "C",
        "Ex": "F#",
        "E#": "F",
        "E": "E",
        "Eb": "D#",
        "E𝄫": "D",
        "Fx": "G",
        "F#": "Gb",
        "F": "F",
        "Fb": "E",
        "F𝄫": "Eb",
        "Gx": "A",
        "G#": "Ab",
        "G": "G",
        "Gb": "F#",
        "G𝄫": "F",
    }

    def test_get_enharmonic(self):
        """
        Test the get_enharmonic method of the Enharmonic class.

        This method tests whether the get_enharmonic method of the Enharmonic
        class correctly returns the enharmonic note value for each note in the
        _ENHARMONIC_MAPPING dictionary. For each note in the dictionary, a
        ScientificPitchNotation object is created with the note and the default
        octave, and passed to the Enharmonic constructor. The get_enharmonic
        method is then called on the Enharmonic object, and the pitch of the
        returned note is compared to the expected enharmonic value in the
        dictionary using self.assertEqual.

        Parameters
        ----------
        self : unittest.TestCase
            An instance of the unittest.TestCase class.

        Returns
        -------
        None
            This method does not return anything. It raises an exception if the
            test fails.

        Raises
        ------
        AssertionError
            If the pitch of the returned note is not equal to the expected
            enharmonic value.
        """
        for note, enharmonic_value in self._ENHARMONIC_MAPPING.items():
            spn = ScientificPitchNotation(note, DEFAULT_OCTAVE)
            enharmonic_note = Enharmonic(spn).get_enharmonic()
            self.assertEqual(enharmonic_note.pitch, enharmonic_value)

    def test_is_enharmonic(self):
        """
        Test for the `is_enharmonic` method of the Enharmonic class.

        Tests if the method returns True for each note in the
        `_ENHARMONIC_MAPPING` dictionary. For each note in the dictionary, a
        `ScientificPitchNotation` object is created with the note and the
        default octave, and passed to the `Enharmonic` constructor. The
        `is_enharmonic` method is then called on the `Enharmonic` object, and
        the result is compared to True using `self.assertTrue`.

        Returns
        -------
        None

        """
        for pitch in self._ENHARMONIC_MAPPING.keys():
            spn = ScientificPitchNotation(pitch, DEFAULT_OCTAVE)
            self.assertTrue(Enharmonic(spn).is_enharmonic(), True)

    def test_get_all_enharmonics(self):
        """
        Test the `get_all_enharmonics()` method of the `Enharmonic` class.

        This method checks if the `get_all_enharmonics()` method returns the
        expected list of enharmonic notes for a given note.

        Raises:
            AssertionError: If the returned list of enharmonic notes is not
            equal to the expected list ['A𝄫', 'Fx', 'G'].
        """
        self.assertEqual(
            Enharmonic(
                ScientificPitchNotation("G", DEFAULT_OCTAVE)
            ).get_all_enharmonics(),
            ["A𝄫", "Fx", "G"],
        )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
# Name:         test_note_frequency_converter.py
# Purpose:      Pymuco Unit Tests
#
# Authors:      German Margon
#
# Copyright:    Copyright © 2022-2023 German Margon. All rights reserved.
# License:      BSD 3-Clause License, see LICENSE
# ------------------------------------------------------------------------------
import re
import unittest

from pymuco import NoteFrequencyConverter, ScientificPitchNotation


class TestNoteFrequencyConverter(unittest.TestCase):
    """
    A test case class used for testing the functionality of a note frequency
    converter.

    Attributes:
    _MIDI_NOTES (dict): A dictionary that stores the MIDI notes and their
    corresponding frequencies.

    Methods:
    setUp(self): Initializes a NoteFrequencyConverter object.
    test_get_frequency_from_midi_number(self): Test the
    get_frequency_from_midi_number method of the Converter class with valid
    input.
    test_get_frequency_from_note_name(self): Test the
    get_frequency_from_note_name method of a Converter object.
    """

    _MIDI_NOTES = {
        127: 12543.85,
        126: 11839.82,
        125: 11175.30,
        124: 10548.08,
        123: 9956.06,
        122: 9397.27,
        121: 8869.84,
        120: 8372.02,
        119: 7902.13,
        118: 7458.62,
        117: 7040.00,
        116: 6644.88,
        115: 6271.93,
        114: 5919.91,
        113: 5587.65,
        112: 5274.04,
        111: 4978.03,
        110: 4698.64,
        109: 4434.92,
        108: 4186.01,
        107: 3951.07,
        106: 3729.31,
        105: 3520.00,
        104: 3322.44,
        103: 3135.96,
        102: 2959.96,
        101: 2793.83,
        100: 2637.02,
        99: 2489.02,
        98: 2349.32,
        97: 2217.46,
        96: 2093.00,
        95: 1975.53,
        94: 1864.66,
        93: 1760.00,
        92: 1661.22,
        91: 1567.98,
        90: 1479.98,
        89: 1396.91,
        88: 1318.51,
        87: 1244.51,
        86: 1174.66,
        85: 1108.73,
        84: 1046.50,
        83: 987.77,
        82: 932.33,
        81: 880.00,
        80: 830.61,
        79: 783.99,
        78: 739.99,
        77: 698.46,
        76: 659.26,
        75: 622.25,
        74: 587.33,
        73: 554.37,
        72: 523.25,
        71: 493.88,
        70: 466.16,
        69: 440.00,
        68: 415.30,
        67: 392.00,
        66: 369.99,
        65: 349.23,
        64: 329.63,
        63: 311.13,
        62: 293.66,
        61: 277.18,
        60: 261.63,
        59: 246.94,
        58: 233.08,
        57: 220.00,
        56: 207.65,
        55: 196.00,
        54: 185.00,
        53: 174.61,
        52: 164.81,
        51: 155.56,
        50: 146.83,
        49: 138.59,
        48: 130.81,
        47: 123.47,
        46: 116.54,
        45: 110.00,
        44: 103.83,
        43: 98.00,
        42: 92.50,
        41: 87.31,
        40: 82.41,
        39: 77.78,
        38: 73.42,
        37: 69.30,
        36: 65.41,
        35: 61.74,
        34: 58.27,
        33: 55.00,
        32: 51.91,
        31: 49.00,
        30: 46.25,
        29: 43.65,
        28: 41.20,
        27: 38.89,
        26: 36.71,
        25: 34.65,
        24: 32.70,
        23: 30.87,
        22: 29.14,
        21: 27.50,
        20: 25.96,
        19: 24.50,
        18: 23.12,
        17: 21.83,
        16: 20.60,
        15: 19.45,
        14: 18.35,
        13: 17.32,
        12: 16.35,
        11: 15.43,
        10: 14.57,
        9: 13.75,
        8: 12.98,
        7: 12.25,
        6: 11.56,
        5: 10.91,
        4: 10.30,
        3: 9.72,
        2: 9.18,
        1: 8.66,
        0: 8.18,
    }

    _NOTES = {
        127: "G9",
        126: "F#9/Gb9",
        125: "F9",
        124: "E9",
        123: "D#9/Eb9",
        122: "D9",
        121: "C#9/Db9",
        120: "C9",
        119: "B8",
        118: "A#8/Bb8",
        117: "A8",
        116: "G#8/Ab8",
        115: "G8",
        114: "F#8/Gb8",
        113: "F8",
        112: "E8",
        111: "D#8/Eb8",
        110: "D8",
        109: "C#8/Db8",
        108: "C8",
        107: "B7",
        106: "A#7/Bb7",
        105: "A7",
        104: "G#7/Ab7",
        103: "G7",
        102: "F#7/Gb7",
        101: "F7",
        100: "E7",
        99: "D#7/Eb7",
        98: "D7",
        97: "C#7/Db7",
        96: "C7",
        95: "B6",
        94: "A#6/Bb6",
        93: "A6",
        92: "G#6/Ab6",
        91: "G6",
        90: "F#6/Gb6",
        89: "F6",
        88: "E6",
        87: "D#6/Eb6",
        86: "D6",
        85: "C#6/Db6",
        84: "C6",
        83: "B5",
        82: "A#5/Bb5",
        81: "A5",
        80: "G#5/Ab5",
        79: "G5",
        78: "F#5/Gb5",
        77: "F5",
        76: "E5",
        75: "D#5/Eb5",
        74: "D5",
        73: "C#5/Db5",
        72: "C5",
        71: "B4",
        70: "A#4/Bb4",
        69: "A4",
        68: "G#4/Ab4",
        67: "G4",
        66: "F#4/Gb4",
        65: "F4",
        64: "E4",
        63: "D#4/Eb4",
        62: "D4",
        61: "C#4/Db4",
        60: "C4",
        59: "B3",
        58: "A#3/Bb3",
        57: "A3",
        56: "G#3/Ab3",
        55: "G3",
        54: "F#3/Gb3",
        53: "F3",
        52: "E3",
        51: "D#3/Eb3",
        51: "D#3/Eb3",
        50: "D3",
        49: "C#3/Db3",
        48: "C3",
        47: "B2",
        46: "A#2/Bb2",
        45: "A2",
        44: "G#2/Ab2",
        43: "G2",
        42: "F#2/Gb2",
        41: "F2",
        40: "E2",
        39: "D#2/Eb2",
        38: "D2",
        37: "C#2/Db2",
        36: "C2",
        35: "B1",
        34: "A#1/Bb1",
        33: "A1",
        32: "G#1/Ab1",
        31: "G1",
        30: "F#1/Gb1",
        29: "F1",
        28: "E1",
        27: "D#1/Eb1",
        26: "D1",
        25: "C#1/Db1",
        24: "C1",
        23: "B0",
        22: "A#0/Bb0",
        21: "A0",
    }

    def setUp(self):
        """
        Initializes a NoteFrequencyConverter object.

        Returns:
        None.
        """
        self.converter = NoteFrequencyConverter()

    def test_get_frequency_from_midi_number(self):
        """
        Test the get_frequency_from_midi_number method of the Converter class
        with valid input.

        This method iterates through a dictionary `_MIDI_NOTES` which maps MIDI
        note numbers to their corresponding frequency values. For each key in
        the dictionary, the method calls the `get_frequency_from_midi_number`
        method of the `Converter` class and compares the returned value to the
        expected frequency value.

        Raises:
            AssertionError: if the returned frequency value does not match the
            expected frequency value for any MIDI note number.

        Returns:
            None. This method asserts the correctness of the
            `get_frequency_from_midi_number` method.

        Example:
            >>> converter = Converter()
            >>> converter.test_get_frequency_from_midi_number()
            None
        """
        for key, value in self._MIDI_NOTES.items():
            self.assertEqual(
                value, self.converter.get_frequency_from_midi_number(key)
            )

    def test_get_frequency_from_note_name(self):
        """
        Test the get_frequency_from_note_name method of a Converter object.

        This method loops through a dictionary of note names and their
        corresponding MIDI numbers. For each note name, the method parses the
        note name to extract the pitch and octave. The method then creates a
        `ScientificPitchNotation` object from the parsed pitch and octave, and
        calls the `get_frequency_from_note_name` method of the `Converter`
        object with this object. The method asserts that the returned frequency
        value matches the expected MIDI number.

        Raises:
            AssertionError: if the returned frequency value does not match the
            expected MIDI number for any note name.

        Returns:
            None. This method asserts the correctness of the
            `get_frequency_from_note_name` method.

        Example:
            >>> converter = Converter()
            >>> converter.test_get_frequency_from_note_name()
            None
        """
        number_pattern = r"\d+\.?\d*"
        pitch_pattern = r"[a-zA-Z]+"

        for key1, key2 in zip(self._NOTES, self._MIDI_NOTES):
            number_match = re.search(number_pattern, self._NOTES[key1])
            octave = int(number_match.group())

            pitch_match = re.findall(pitch_pattern, self._NOTES[key1])
            pitch = "".join(pitch_match)

            if len(self._NOTES[key1]) > 2:
                continue

            self.assertEqual(
                self.converter.get_frequency_from_note_name(
                    ScientificPitchNotation(pitch, octave)
                ),
                self._MIDI_NOTES[key2],
            )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
# Name:         test_note_duration.py
# Purpose:      Pymuco Unit Tests
#
# Authors:      German Margon
#
# Copyright:    Copyright © 2022-2023 German Margon. All rights reserved.
# License:      BSD 3-Clause License, see LICENSE
# ------------------------------------------------------------------------------
import unittest

from pymuco import NoteDuration, NoteDurationMapping


class TestNoteDurationMapping(unittest.TestCase):
    """
    A unittest test case to test the NoteDurationMapping class.

    Attributes
    ----------
    expected_note_lengths : list of tuples
        A list of tuples representing the expected note lengths for each note
        type. Each tuple contains a string representing the note type and a
        float representing its duration in terms of whole notes.

    Methods
    -------
    setUp()
        Set up the test fixture by creating an instance of NoteDurationMapping
        and assigning it to the instance variable self.note_duration_mapping.
    test_note_duration_mapping()
        Test the NoteDurationMapping.get_note_duration method by comparing its
        output with the expected_note_lengths list.
    """

    expected_note_lengths = [
        ("WHOLE_NOTE", 1.0),
        ("HALF_NOTE", 0.5),
        ("QUARTER_NOTE", 0.25),
        ("EIGHTH_NOTE", 0.125),
        ("SIXTEENTH_NOTE", 0.0625),
        ("THIRTY_SECOND_NOTE", 0.03125),
    ]

    def setUp(self):
        """
        Set up the test fixture.

        This method creates an instance of NoteDurationMapping and assigns it
        to the instance variable self.note_duration_mapping.

        Returns
        -------
        None
            This method only sets up the test fixture and doesn't return
            anything.
        """
        self.note_duration_mapping = NoteDurationMapping()

    def test_note_duration_mapping(self):
        """
        Test NoteDurationMapping.get_note_duration method.

        Returns
        -------
        None
            This function only tests that the output of get_note_duration method
            matches the expected_note_lengths list and raises an AssertionError
            otherwise.

        Raises
        ------
        AssertionError
            If the output of get_note_duration method doesn't match the
            expected_note_lengths list.
        """
        self.assertListEqual(
            self.note_duration_mapping.get_note_duration(),
            self.expected_note_lengths,
        )


class TestNoteDuration(unittest.TestCase):
    """
    A class for testing the NoteDuration class.

    Methods
    -------
    test_duration_type()
        Test that duration_type() method returns the expected value for each
        input.
    test_duration_length()
        Test that duration_length() method returns the expected value for each
        input.

    Raises
    ------
    ValueError
        If an invalid note type or length is provided.
    """

    def test_duration_type(self):
        """
        Test that duration_type() method returns the expected value for each
        input.

        Returns
        -------
        None

        Raises
        ------
        ValueError
            If an invalid note type is provided.
        """
        self.assertEqual(
            NoteDuration("WHOLE_NOTE").duration_type, "WHOLE_NOTE"
        )
        self.assertEqual(NoteDuration("HALF_NOTE").duration_type, "HALF_NOTE")
        self.assertEqual(
            NoteDuration("QUARTER_NOTE").duration_type, "QUARTER_NOTE"
        )
        self.assertEqual(
            NoteDuration("EIGHTH_NOTE").duration_type, "EIGHTH_NOTE"
        )
        self.assertEqual(
            NoteDuration("SIXTEENTH_NOTE").duration_type, "SIXTEENTH_NOTE"
        )
        self.assertEqual(
            NoteDuration("THIRTY_SECOND_NOTE").duration_type,
            "THIRTY_SECOND_NOTE",
        )
        with self.assertRaises(ValueError):
            NoteDuration("INVALID_NOTE_TYPE")

    def test_duration_length(self):
        """
        Test that duration_length() method returns the expected value for each
        input.

        Returns
        -------
        None

        Raises
        ------
        ValueError
            If an invalid note length is provided.
        """
        self.assertAlmostEqual(NoteDuration(1.0).duration_length, 1.0)
        self.assertAlmostEqual(NoteDuration(0.5).duration_length, 0.5)
        self.assertAlmostEqual(NoteDuration(0.25).duration_length, 0.25)
        self.assertAlmostEqual(NoteDuration(0.125).duration_length, 0.125)
        self.assertAlmostEqual(NoteDuration(0.0625).duration_length, 0.0625)
        self.assertAlmostEqual(NoteDuration(0.03125).duration_length, 0.03125)
        with self.assertRaises(ValueError):
            NoteDuration(2.0).duration_length



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
# Name:         Chord.py
# Purpose:      Pymuco classes
#
# Authors:      German Margon
#
# Copyright:    Copyright © 2022-2023 German Margon. All rights reserved.
# License:      BSD 3-Clause License, see LICENSE
# ------------------------------------------------------------------------------
from enum import Enum

from .CircleOfFifths import CircleOfFifths
from .Enharmonic import Enharmonic
from .Interval import IntervalCalculator, IntervalName
from .KeySignature import KeySignature
from .MusicData import DEFAULT_OCTAVE, m
from .ScientificPitchNotation import ScientificPitchNotation


class ChordType(Enum):
    """
    An enumeration class that represents the type of a chord.

    Attributes:
    -----------
        MAJOR (Tuple[IntervalName]): A major chord represented by a tuple of
        intervals, consisting of a major third and perfect fifth.
        MINOR (Tuple[IntervalName]): A minor chord represented by a tuple of
        intervals, consisting of a minor third and perfect fifth.
        DIMINISHED (Tuple[IntervalName]): A diminished chord represented by a
        tuple of intervals, consisting of a minor third and tritone.
        AUGMENTED (Tuple[IntervalName]): An augmented chord represented by a
        tuple of intervals, consisting of a major third and minor sixth.
        MAJOR_SEVENTH (Tuple[IntervalName]): A major seventh chord represented
        by a tuple of intervals, consisting of a major third, perfect fifth,
        and major seventh.
        MINOR_SEVENTH (Tuple[IntervalName]): A minor seventh chord represented
        by a tuple of intervals, consisting of a minor third, perfect fifth,
        and minor seventh.
        DOMINANT_SEVENTH (Tuple[IntervalName]): A dominant seventh chord
        represented by a tuple of intervals, consisting of a major third,
        perfect fifth, and minor seventh.
        HALF_DIMINISHED (Tuple[IntervalName]): A half-diminished chord
        represented by a tuple of intervals, consisting of a minor third,
        tritone, and minor seventh.
        SUSPENDED_SECOND (Tuple[IntervalName]): A suspended second chord
        represented by a tuple of intervals, consisting of a major second and
        perfect fifth.
        SUSPENDED_FOURTH (Tuple[IntervalName]): A suspended fourth chord
        represented by a tuple of intervals, consisting of a perfect fourth
        and perfect fifth.
    """

    MAJOR = (IntervalName.MAJOR_THIRD, IntervalName.PERFECT_FIFTH)
    MINOR = (IntervalName.MINOR_THIRD, IntervalName.PERFECT_FIFTH)
    DIMINISHED = (IntervalName.MINOR_THIRD, IntervalName.TRITONE)
    AUGMENTED = (IntervalName.MAJOR_THIRD, IntervalName.MINOR_SIXTH)
    MAJOR_SEVENTH = (
        IntervalName.MAJOR_THIRD,
        IntervalName.PERFECT_FIFTH,
        IntervalName.MAJOR_SEVENTH,
    )
    MINOR_SEVENTH = (
        IntervalName.MINOR_THIRD,
        IntervalName.PERFECT_FIFTH,
        IntervalName.MINOR_SEVENTH,
    )
    DOMINANT_SEVENTH = (
        IntervalName.MAJOR_THIRD,
        IntervalName.PERFECT_FIFTH,
        IntervalName.MINOR_SEVENTH,
    )
    HALF_DIMINISHED = (
        IntervalName.MINOR_THIRD,
        IntervalName.TRITONE,
        IntervalName.MINOR_SEVENTH,
    )
    SUSPENDED_SECOND = (IntervalName.MAJOR_SECOND, IntervalName.PERFECT_FIFTH)
    SUSPENDED_FOURTH = (
        IntervalName.PERFECT_FOURTH,
        IntervalName.PERFECT_FIFTH,
    )


class Chord:
    """
    Represents a musical chord, defined by a root note and a chord type.

    Attributes
    ----------
    _CIRCLE_OF_FIFTHS : list of str
        A list of notes in the circle of fifths.
    _RELATIVE_MINORS : list of str
        A list of relative minor notes.

    Methods
    -------
    __init__(root_note, chord_type)
        Initializes a new Chord object with the specified root note and chord
        type.

        Parameters
        ----------
        root_note : ScientificPitchNotation
            The root note of the chord.
        chord_type : ChordType
            The type of the chord.

    get_chord_notes()
        Returns the notes of the chord represented by this object.

        Returns
        -------
        list of str
            The notes of the chord.

    _calculate_chord()
        Calculates the notes of the chord represented by this object.

    _calculate_chord_notes()
        Calculates the notes of the chord based on its root note and chord
        type.
    """

    _CIRCLE_OF_FIFTHS = CircleOfFifths().circle_of_fifths
    _RELATIVE_MINORS = CircleOfFifths().relative_minors

    def __init__(
        self, root_note: ScientificPitchNotation, chord_type: ChordType
    ):
        """
        Initializes a new Chord object with the specified root note and chord
        type.

        Args:
        -----
        - root_note: The root note of the chord, specified as a
        ScientificPitchNotation object.
        - chord_type: The type of chord, specified as a ChordType enumeration
        value.
        """
        self._root_note = root_note
        self._chord_type = chord_type

    def get_chord_notes(self):
        """
        Returns the notes of the chord represented by this object.
        If the root note is a relative minor or a note in the circle of fifths,
        returns the chord as calculated by _calculate_chord(). Otherwise,
        returns the chord notes as calculated by _calculate_chord_notes().
        """
        if self._root_note.pitch not in (
            self._RELATIVE_MINORS + self._CIRCLE_OF_FIFTHS
        ):
            return self._calculate_chord_notes()
        else:
            return self._calculate_chord()

    def _calculate_chord(self):
        """
        Calculates the notes of the chord represented by this object by first
        calculating the chord notes using the _calculate_chord_notes method,
        and then transposing them to match the key signature of the chord.

        Returns:
        --------
        - A list of the notes of the chord, transposed to match the key
        signature.
        """
        notes = self._calculate_chord_notes()

        chord_type_name = []
        for key in ChordType.__members__:
            chord_type_name.append(key)

        if self._chord_type.name in chord_type_name:
            if self._chord_type.name == "SUSPENDED_FOURTH":
                return self._calculate_chord_notes()

            if (
                KeySignature(
                    self._root_note,
                    self._chord_type.value[0].name.split("_")[0],
                ).get_key_signature_notes()
                is None
            ):
                key_signature_notes = KeySignature(
                    self._root_note, m
                ).get_key_signature_notes()
            else:
                key_signature_notes = KeySignature(
                    self._root_note,
                    self._chord_type.value[0].name.split("_")[0],
                ).get_key_signature_notes()

            if key_signature_notes is None:
                return notes

            for note in notes:
                for key in key_signature_notes:
                    if (
                        Enharmonic(
                            ScientificPitchNotation(note, DEFAULT_OCTAVE)
                        ).is_enharmonic()
                        is True
                    ):
                        if (
                            Enharmonic(
                                ScientificPitchNotation(note, DEFAULT_OCTAVE)
                            )
                            .get_enharmonic()
                            .pitch
                            == key
                        ):
                            notes[notes.index(note)] = key

        return notes

    def _calculate_chord_notes(self):
        """Calculates the notes of the chord based on its root note and chord
        type.

        Returns:
        --------
            A list of Note objects representing the notes of the chord.
        """
        interval = IntervalCalculator()
        notes = [self._root_note.pitch] + [
            interval.calculate_ascending_interval(
                interval_name, self._root_note
            )
            for interval_name in self._chord_type.value
        ]
        return notes



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
# Name:         test_audio_converter.py
# Purpose:      Pymuco Unit Tests
#
# Authors:      German Margon
#
# Copyright:    Copyright © 2022-2023 German Margon. All rights reserved.
# License:      BSD 3-Clause License, see LICENSE
# ------------------------------------------------------------------------------
import os
import unittest
import wave
from unittest.mock import MagicMock

from pymuco import AudioConverter, ScientificPitchNotation


class TestAudioConverter(unittest.TestCase):
    """
    Test case for the AudioConverter class.

    This test case contains four test methods that test the
    `_validate_file_path()` method of the AudioConverter class. Each test
    method simulates a different scenario in which the file path is either
    valid or invalid and asserts that the `_validate_file_path()` method
    behaves correctly in each case.

    Methods:
    ----------
    setUp(self):
        Method called to prepare the test fixture. This method creates an
        instance of the AudioConverter class.

    test_validate_file_path_with_valid_path(self):
        Test that the `validate_file_path()` method of the AudioConverter class
        correctly validates a file path that exists and has the necessary
        permissions. This test mocks the `os.path.exists()` and `os.access()`
        methods to return True, indicating that the file path is valid.

    test_validate_file_path_with_nonexistent_path(self):
        Test that the `_validate_file_path()` method returns False if the
        specified file path does not exist. The test mocks the
        `os.path.exists()` method to always return False for the duration of
        the test and calls the `_validate_file_path()` method with a mock file
        path. The test uses the `assertRaises()` method of the unittest module
        to check that a `ValueError` is raised.

    test_validate_file_path_with_unreadable_path(self):
        Test that the `_validate_file_path()` method of the AudioConverter
        class raises a `ValueError` if the file path is not readable. The test
        mocks the `os.path.exists()` and `os.access()` methods to simulate a
        readable directory with an unreadable file. It then calls the
        `_validate_file_path()` method of an instance of AudioConverter. Since
        the file is unreadable, the method should raise a `ValueError`. The
        `assertRaises()` method of the unittest module is used to check that
        the `ValueError` is raised.

    test_validate_file_path_with_unwritable_path(self):
        Test that the `_validate_file_path()` method of the AudioConverter
        class raises a `ValueError` when the specified file path is not
        writable. The method sets up a mock object for `os.path.exists()`
        which returns True and for `os.access()` which returns True and False.
        This simulates a file path that exists but is not writable. The test
        asserts that a `ValueError` is raised when the `_validate_file_path()`
        method is called in this situation.

    The test case inherits from `unittest.TestCase`.
    """

    def setUp(self):
        """
        Set up the unit test fixture by creating an instance of the
        AudioConverter class.
        """
        self.instance = AudioConverter()

    def test_validate_file_path_with_valid_path(self):
        """
        Test that the validate_file_path() method of the AudioConverter class
        correctly validates a file path that exists and has the necessary
        permissions.

        This test mocks the os.path.exists() and os.access() methods to return
        True, indicating that the file path is valid.

        The test sets the file path of the AudioConverter instance to a valid
        path and calls the validate_file_path() method. It verifies that the
        method returns None, indicating that the file path is valid, using the
        assertIsNone() method of the unittest module.

        Raises:
        -------
        AssertionError: If the validate_file_path() method returns a non-None
        value or raises an unexpected exception.
        """
        os.path.exists = MagicMock(return_value=True)
        os.access = MagicMock(return_value=True)
        self.instance._file_path = "/path/to/output"
        self.assertIsNone(self.instance._validate_file_path())

    def test_validate_file_path_with_nonexistent_path(self):
        """
        Test whether the `validate_file_path()` method of the `AudioConverter`
        class returns `False` when the specified file path does not exist.

        The test creates a mock file path and mocks the `os.path.exists()`
        method to always return `False`. It then calls the
        `validate_file_path()` method with the mock file path and asserts that
        the method returns `False` using the `assertFalse()` method of the
        `unittest` module.

        Raises:
        -------
        ValueError:
            If the file path exists when it should not.

        """
        os.path.exists = MagicMock(return_value=False)
        with self.assertRaises(ValueError):
            self.instance._validate_file_path()

    def test_validate_file_path_with_unreadable_path(self):
        """
        Test that the `_validate_file_path()` method of the `AudioConverter`
        class raises a `ValueError` if the specified file path is not readable.

        This test mocks the `os.path.exists()` and `os.access()` methods to
        simulate a readable directory with an unreadable file. It then calls
        the `_validate_file_path()` method of an instance of `AudioConverter`.
        Since the file is unreadable, the method should raise a `ValueError`.
        The `assertRaises()` method of the `unittest` module is used to check
        that the `ValueError` is raised.

        Raises:
        -------
            ValueError: If the file path is not readable.
        """
        os.path.exists = MagicMock(return_value=True)
        os.access = MagicMock(side_effect=[False, True])
        with self.assertRaises(ValueError):
            self.instance._validate_file_path()

    def test_validate_file_path_with_unwritable_path(self):
        """
        Test that the _validate_file_path() method of the AudioConverter class
        raises a ValueError when the specified file path is not writable.

        The test sets up a mock object for os.path.exists() which returns True
        and for os.access() which returns True and False in sequence. This
        simulates a file path that exists but is not writable.
        The _validate_file_path() method of an instance of AudioConverter is
        then called. Since the file is not writable, the method should raise a
        ValueError. The assertRaises() method of the unittest module is used to
        check that the ValueError is raised.

        Raises:
        -------
            ValueError: If the specified file path is not writable.
        """
        os.path.exists = MagicMock(return_value=True)
        os.access = MagicMock(side_effect=[True, False])
        with self.assertRaises(ValueError):
            self.instance._validate_file_path()


class TestGenerateSamples(unittest.TestCase):
    """
    This class contains unit tests for the _generate_samples() method of the
    AudioConverter class using the unittest.TestCase framework.

    Methods:
    -------

    setUp():
    Method called to prepare the test fixture. This creates an instance of the
    `AudioConverter` class used in the individual test methods to test its
    functionality.

    test_sample_list_length():
        Verify that the `_generate_samples()` method generates a list of
        samples of the expected length. This test calls the
        `_generate_samples()` method with a frequency of 440 Hz and a duration
        of 1 second. The expected length of the resulting list of samples is
        44100, which corresponds to the number of samples in 1 second of audio
        with a sample rate of 44100 Hz. This test uses the `assertEqual()`
        method of the `unittest` module to verify that the length of the actual
        list of samples is equal to the expected length.

    test_sample_list_type():
        Verify that the `_generate_samples()` method of the `AudioConverter`
        class returns a list object for the `_sample_list` attribute after
        being called with valid parameters. This test ensures that the data
        type of the returned object is correct to ensure proper functionality
        in future use of this attribute.

    test_sample_list_values():
        Verify the values of the sample list generated by the
        `_generate_samples()` method. This unit test generates a sample list
        with a specified frequency and duration, then checks that the maximum
        and minimum values of the list are within the range of [-32768, 32767].
        This is to ensure that the generated audio samples are within the range
        of a 16-bit PCM audio format, which is commonly used for digital audio.

    Raises:
    -------

    AssertionError: If the maximum or minimum value of the sample list is
    outside the expected range.

    Note:
    -------

    The `_generate_samples()` method is responsible for generating a list of
    audio samples based on the given frequency and duration.
    """

    def setUp(self):
        """
        Method called to prepare the test fixture.

        This method creates an instance of the AudioConverter class that is
        used in the individual test methods to test its functionality.
        """
        self.instance = AudioConverter()

    def test_sample_list_length(self):
        """
        Test case for the "_generate_samples()" method of the AudioConverter
        class.

        This unit test verifies that the method generates a list of samples of
        the expected length. The "_generate_samples()" method is called with a
        frequency of 440 Hz and a duration of 1 second. The expected length of
        the resulting list of samples is 44100, which corresponds to the number
        of samples in 1 second of audio with a sample rate of 44100 Hz.

        The test uses the assertEqual() method of the unittest module to verify
        that the length of the actual list of samples is equal to the expected
        length.
        """
        self.instance._generate_samples(440, 1.0)
        self.assertEqual(len(self.instance._sample_list), 44100)

    def test_sample_list_type(self):
        """
        Test that the "_generate_samples" method of the "AudioConverter" class
        returns a list object for the "_sample_list" attribute after being
        called with valid parameters. This test verifies that the data type of
        the returned object is correct to ensure proper functionality in future
        use of this attribute.
        """
        self.instance._generate_samples(440, 1.0)
        self.assertIsInstance(self.instance._sample_list, list)

    def test_sample_list_values(self):
        """
        Verify values of the sample list generated by `_generate_samples()`
        method.

        This unit test generates a sample list with a specified frequency and
        duration, then checks that the maximum and minimum values of the list
        are within the range of [-32768, 32767]. This is to ensure that the
        generated audio samples are within the range of a 16-bit PCM audio
        format, which is commonly used for digital audio.

        Raises:
        -------
            AssertionError: If the maximum or minimum value of the sample list
            is outside the expected range.
        """
        self.instance._generate_samples(440, 1.0)
        max_value = max(self.instance._sample_list)
        min_value = min(self.instance._sample_list)
        self.assertLessEqual(max_value, 32767)
        self.assertGreaterEqual(min_value, -32768)


class TestWriteToFile(unittest.TestCase):
    """
    This class contains unit tests for the "_write_to_file()" method of the
    AudioConverter class.

    Attributes:
    -----------
    Inherits from:
        unittest.TestCase : A unit testing framework.

    Methods:
    --------
    setUp(self):
        This method initializes an instance of the AudioConverter class with a
        sample list. It is called before each test method.

    test_file_created(self):
        Test that the _write_to_file() method of the AudioConverter class
        creates a file with the specified name in the specified directory.
        The test creates an instance of AudioConverter, calls the
        _write_to_file() method, and checks if the file exists
        by joining the file name and directory using os.path.join() and
        checking if the resulting path is a file using os.path.isfile().
        The test uses the assertTrue() method of the unittest module to assert
        that the file exists.

    test_file_params(self):
        Test that the file created by the _write_to_file() method of the
        AudioConverter instance has the correct parameters.
        The method creates a .wav file with a sample rate of 44100 Hz, a bit
        depth of 16 bits, and a single channel.
        The test checks the file parameters using the wave module. It verifies
        that the file has only one channel, a sample width of 2 bytes
        (16 bits), and a sample rate of 44100 Hz. The assertEqual() method is
        used to compare the actual values of the file parameters to the
        expected values.
    """

    def setUp(self):
        """
        Method called to prepare the test fixture.

        This method creates an instance of the `AudioConverter` class that is
        used in the individual test methods to test its functionality. It also
        sets the `_sample_list` attribute of the instance to a predefined list
        of float values [0.5, 0.1, -0.3, -0.7, 0.2]. This list of samples
        is used in some of the unit tests to verify the functionality of other
        methods of the `AudioConverter` class.

        Returns:
        --------
        None
        """
        self.instance = AudioConverter()
        self.instance._sample_list = [0.5, 0.1, -0.3, -0.7, 0.2]

    def test_file_created(self):
        """
        Verify that the `_write_to_file()` method of the `AudioConverter`
        class creates a file with the specified name in the specified
        directory.

        The test creates an instance of `AudioConverter` and calls the
        `_write_to_file()` method. It then checks if the file exists by joining
        the file name and directory using `os.path.join()` and checking if the
        resulting path is a file using `os.path.isfile()`. The test uses the
        `assertTrue()` method of the `unittest` module to assert that the file
        exists.

        Raises:
        -------
        AssertionError: If the file does not exist in the specified directory
        or if any of the underlying methods called during `_write_to_file()`
        execution raises an exception.
        """
        self.instance._write_to_file()
        file_path = os.path.join(self.instance._file_path, self.instance._name)
        self.assertTrue(os.path.isfile(file_path))

    def test_file_params(self):
        """
        This unit test ensures that the file created by the "_write_to_file()"
        method of the AudioConverter instance has the correct parameters.

        The method creates a .wav file with a sample rate of 44100 Hz, a bit
        depth of 16 bits, and a single channel.

        The test checks the file parameters using the wave module. It verifies
        that the file has only one channel, a sample width of 2 bytes
        (16 bits), and a sample rate of 44100 Hz.

        Raises:
        -------
        AssertionError: If any of the file parameters (number of channels,
        sample width, and sample rate) do not match the expected values.
        """
        self.instance._write_to_file()
        file_path = os.path.join(self.instance._file_path, self.instance._name)
        with wave.open(file_path, "r") as file:
            self.assertEqual(file.getnchannels(), 1)
            self.assertEqual(file.getsampwidth(), 2)
            self.assertEqual(file.getframerate(), 44100)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
# Name:         Tonality.py
# Purpose:      Pymuco classes
#
# Authors:      German Margon
#
# Copyright:    Copyright © 2022-2023 German Margon. All rights reserved.
# License:      BSD 3-Clause License, see LICENSE
# ------------------------------------------------------------------------------
from .ScientificPitchNotation import ScientificPitchNotation


class Tonality:
    """
    A class representing a musical tonality, defined by a root note and a
    tonality type.

    Parameters
    ----------
    root_note : ScientificPitchNotation
        The root note of the tonality, specified as a ScientificPitchNotation
        object.
    tonality_type : str
        The type of the tonality, such as "major" or "minor".

    Attributes
    ----------
    _root_note : ScientificPitchNotation
        The root note of the tonality.
    _tonality_type : str
        The type of the tonality.
    """

    def __init__(self, root_note: ScientificPitchNotation, tonality_type):
        self._root_note = root_note
        self._tonality_type = tonality_type



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
# Name:         test_midi_utils.py
# Purpose:      Pymuco Unit Tests
#
# Authors:      German Margon
#
# Copyright:    Copyright © 2022-2023 German Margon. All rights reserved.
# License:      BSD 3-Clause License, see LICENSE
# ------------------------------------------------------------------------------
import re
import unittest

from pymuco import MIDINoteConverter, ScientificPitchNotation


class TestMIDINoteConverter(unittest.TestCase):
    """
    TestMIDINoteConverter is a class that contains unit tests for the
    note_name_to_midi_note method of the MIDINoteConverter class.

    Attributes
    ----------
    _NOTES : dict
        A dictionary of MIDI notes and their corresponding scientific pitch
        notation.

    Methods
    -------
    test_note_name_to_midi_note():
        Tests the note_name_to_midi_note method by looping through the _NOTES
        dictionary and asserting that the calculated MIDI value of each note is
        equal to the expected MIDI value.

    Notes
    -----
    This class is likely part of a larger test suite that tests the
    functionality of the MIDINoteConverter class.
    """

    _NOTES = {
        127: "G9",
        126: "F#9/Gb9",
        125: "F9",
        124: "E9",
        123: "D#9/Eb9",
        122: "D9",
        121: "C#9/Db9",
        120: "C9",
        119: "B8",
        118: "A#8/Bb8",
        117: "A8",
        116: "G#8/Ab8",
        115: "G8",
        114: "F#8/Gb8",
        113: "F8",
        112: "E8",
        111: "D#8/Eb8",
        110: "D8",
        109: "C#8/Db8",
        108: "C8",
        107: "B7",
        106: "A#7/Bb7",
        105: "A7",
        104: "G#7/Ab7",
        103: "G7",
        102: "F#7/Gb7",
        101: "F7",
        100: "E7",
        99: "D#7/Eb7",
        98: "D7",
        97: "C#7/Db7",
        96: "C7",
        95: "B6",
        94: "A#6/Bb6",
        93: "A6",
        92: "G#6/Ab6",
        91: "G6",
        90: "F#6/Gb6",
        89: "F6",
        88: "E6",
        87: "D#6/Eb6",
        86: "D6",
        85: "C#6/Db6",
        84: "C6",
        83: "B5",
        82: "A#5/Bb5",
        81: "A5",
        80: "G#5/Ab5",
        79: "G5",
        78: "F#5/Gb5",
        77: "F5",
        76: "E5",
        75: "D#5/Eb5",
        74: "D5",
        73: "C#5/Db5",
        72: "C5",
        71: "B4",
        70: "A#4/Bb4",
        69: "A4",
        68: "G#4/Ab4",
        67: "G4",
        66: "F#4/Gb4",
        65: "F4",
        64: "E4",
        63: "D#4/Eb4",
        62: "D4",
        61: "C#4/Db4",
        60: "C4",
        59: "B3",
        58: "A#3/Bb3",
        57: "A3",
        56: "G#3/Ab3",
        55: "G3",
        54: "F#3/Gb3",
        53: "F3",
        52: "E3",
        51: "D#3/Eb3",
        51: "D#3/Eb3",
        50: "D3",
        49: "C#3/Db3",
        48: "C3",
        47: "B2",
        46: "A#2/Bb2",
        45: "A2",
        44: "G#2/Ab2",
        43: "G2",
        42: "F#2/Gb2",
        41: "F2",
        40: "E2",
        39: "D#2/Eb2",
        38: "D2",
        37: "C#2/Db2",
        36: "C2",
        35: "B1",
        34: "A#1/Bb1",
        33: "A1",
        32: "G#1/Ab1",
        31: "G1",
        30: "F#1/Gb1",
        29: "F1",
        28: "E1",
        27: "D#1/Eb1",
        26: "D1",
        25: "C#1/Db1",
        24: "C1",
        23: "B0",
        22: "A#0/Bb0",
        21: "A0",
    }

    def test_note_name_to_midi_note(self):
        """
        Test the method note_name_to_midi_note of the MIDINoteConverter class.

        This test loops through a dictionary of notes and their corresponding
        MIDI values, converts each note to its MIDI value using the
        note_name_to_midi_note method, and asserts that the calculated MIDI
        value is equal to the expected MIDI value. The test also asserts that
        the scientific pitch notation of the note is equal to the original note
        value.

        Parameters:
        -----------
        None

        Returns:
        --------
        None

        Notes:
        ------
        - This test uses regular expressions to extract the octave and pitch
        information from the note value.
        - If the pitch is two characters or fewer, the octave and pitch are
        used to create a ScientificPitchNotation object, which is then passed
        to the note_name_to_midi_note method.
        - If the pitch is longer than two characters, the note value is split
        into two parts (before and after the slash) and each part is used to
        create a separate ScientificPitchNotation object, which is then passed
        to the note_name_to_midi_note method.

        Raises:
        -------
        AssertionError:
            If a calculated MIDI value is not equal to the expected MIDI value.

        Examples:
        ---------
        >>> test_note_name_to_midi_note()
        """
        number_pattern = r"\d+\.?\d*"
        pitch_pattern = r"[a-zA-Z]+"

        for key, value in self._NOTES.items():
            number_match = re.search(number_pattern, value)
            octave = int(number_match.group())

            pitch_match = re.findall(pitch_pattern, value)
            pitch = "".join(pitch_match)

            if len(pitch) <= 2:
                spn = ScientificPitchNotation(pitch, octave)
                midi = MIDINoteConverter().note_name_to_midi_note(spn)

                self.assertEqual(key, midi)
                self.assertEqual(value, spn)

            elif len(pitch) > 2:
                spn1 = re.search(r"([^0-9/]+)", value)
                note1 = ScientificPitchNotation(spn1.group(1), octave)
                midi1 = MIDINoteConverter().note_name_to_midi_note(note1)
                self.assertEqual(key, midi1)

                spn2 = re.search(r"(?<=/)[a-zA-Z]+", value)
                note2 = ScientificPitchNotation(spn2.group(), octave)
                midi2 = MIDINoteConverter().note_name_to_midi_note(note2)
                self.assertEqual(key, midi2)

    def test_midi_note_to_note_name(self):
        """
        Test the `midi_note_to_note_name` method of the MIDINoteConverter
        class.

        This unit test method loops through the dictionary of notes and their
        corresponding MIDI values, converts each MIDI value to its
        corresponding note name using the `midi_note_to_note_name` method, and
        asserts that the calculated note name is equal to the expected note
        name.

        Parameters:
        -----------
        self: TestMIDINoteConverter
            An instance of the TestMIDINoteConverter class.

        Returns:
        --------
        None

        Raises:
        -------
        AssertionError
            If the calculated note name is not equal to the expected note name.

        Notes:
        ------
        This test uses regular expressions to extract the pitch information
        from the original note value. The extracted pitch is then compared to
        the calculated note name using the `assertEqual` method. This test
        ensures that the `midi_note_to_note_name` method correctly converts
        MIDI values to their corresponding note names.
        """
        for key, value in self._NOTES.items():
            midi = MIDINoteConverter().midi_note_to_note_name(key)
            spn_value = re.search(r"([^/]+)", value)
            self.assertEqual(spn_value.group(), midi)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
# Name:         ScientificPitchNotation.py
# Purpose:      Pymuco classes
#
# Authors:      German Margon
#
# Copyright:    Copyright © 2022-2023 German Margon. All rights reserved.
# License:      BSD 3-Clause License, see LICENSE
# ------------------------------------------------------------------------------
import re
from typing import Union

from pymuco.MusicData import ACCIDENTALS, NOTE_NAMES


class ScientificPitchNotation:
    """
    The ScientificPitchNotation class represents a musical note in Western
    music theory using the scientific pitch notation format.

    Parameters
    ----------
    pitch : str
    The pitch of the note.
    octave : int
    The octave of the note.

    Attributes
    ----------
    _pitch : str
    The pitch of the note.
    _octave : int
    The octave of the note.

    Methods
    ----------
    init(self, pitch: str, octave: int) -> None
    Initializes a new instance of the class with the specified pitch
    and octave values.
    Validates the input using _validate_params method.
    _validate_params(self) -> None
    Validates the pitch and octave values of a note.
    If invalid, raises a ValueError with a descriptive error message.
    _is_valid_pitch(self) -> None
    Checks whether the pitch of a note is valid.
    If invalid, raises a ValueError with a descriptive error message.
    _is_valid_octave(self) -> None
    Checks whether the octave of a note is valid.
    If invalid, raises a ValueError with a descriptive error message.
    pitch(self) -> str
    Returns the pitch of the note.
    octave(self) -> int
    Returns the octave of the note.
    octave(self, octave: int) -> None
    Setter method to set the octave of the note.
    If the input is not an integer, raises a TypeError with a descriptive
    error message.
    spn(self) -> str
    Returns the scientific pitch notation of the note.
    is_valid_pitch(pitch: str) -> bool
    Checks if a pitch string is valid.
    Returns True if the pitch is a valid note name followed by an
    optional accidental.
    is_valid_octave(octave: int, min_octave: int = 0,
    max_octave: int = 10) -> bool
    Checks if an octave value is valid.
    Returns True if the octave is between min_octave and max_octave, inclusive.
    str(self) -> str
    Returns a string representation of the object.
    eq(self, other: Union['ScientificPitchNotation', str]) -> bool
    Compares two objects of the class.
    Returns True if they represent the same note, False otherwise.

    Notes
    -----
    The is_valid_pitch and is_valid_octave methods use the NOTE_NAMES and
    ACCIDENTALS constants that are assumed to be defined elsewhere in the code.

    Returns
    -------
    None

    Raises
    ------
    ValueError
    If the input is not valid.
    TypeError
    If the input is not an integer.
    """

    def __init__(self, pitch: str, octave: int) -> None:
        self._pitch: str = pitch
        self._octave: int = octave
        self._validate_params()

    @property
    def pitch(self) -> str:
        """Return the pitch of the note"""
        return self._pitch

    @property
    def octave(self) -> int:
        """Return the octave of the note"""
        return self._octave

    @octave.setter
    def octave(self, octave: int) -> None:
        if not isinstance(octave, int):
            raise TypeError("Octave must be an integer.")
        self._octave = octave

    @property
    def spn(self) -> str:
        """
        Return the scientific pitch notation of the note.

        Returns:
        --------
        str
            The scientific pitch notation of the note.
        """
        return f"{self.pitch}{self.octave}"

    @staticmethod
    def is_valid_pitch(pitch: str) -> bool:
        """
        Check if a pitch string is a valid note name followed by an optional
        accidental.

        Args:
        -----
            pitch (str): The pitch string to check.

        Returns:
        --------
            bool: True if the pitch is a valid note name followed by an
            optional accidental.

        Raises:
        -------
            ValueError: If the pitch is not a string or if it does not match
            the expected format (i.e., a note name followed by an optional
            accidental).
        """
        pattern = re.compile(
            f"^[{''.join(NOTE_NAMES)}][{''.join(ACCIDENTALS)}]?$"
        )
        return pattern.match(pitch) is not None

    @staticmethod
    def is_valid_octave(
        octave: int, min_octave: int = 0, max_octave: int = 10
    ) -> bool:
        """
        Check if an octave value is valid.

        Parameters
        ----------
        octave : int
            The octave value to check.
        min_octave : int, optional
            The minimum valid octave (default is 0).
        max_octave : int, optional
            The maximum valid octave (default is 10).

        Returns
        -------
        bool
            True if the octave is between min_octave and max_octave, inclusive.

        """
        return octave >= min_octave and octave <= max_octave

    def _validate_params(self) -> None:
        """
        Validate the pitch and octave values of a note.

        This method checks that the pitch and octave of a note are valid
        according to the conventions of Western music theory.
        If the pitch or octave is invalid, a ValueError is raised with a
        descriptive error message.

        Raises:
        -------
            ValueError: If the pitch or octave is out of range or has an
                invalid format.

        """
        self._is_valid_pitch()
        self._is_valid_octave()

    def _is_valid_pitch(self) -> None:
        """
        Check whether the pitch of a note is valid.

        This method checks whether the pitch of a note is valid according to
        the conventions of Western music theory.
        If the pitch is invalid, a ValueError is raised with a descriptive
        error message.

        Raises:
        -------
            ValueError: If the pitch is out of range or has an invalid format.

        """
        if not self.is_valid_pitch(self._pitch):
            raise ValueError("Invalid pitch.")

    def _is_valid_octave(self) -> None:
        """
        Check whether the octave of a note is valid.

        This method checks whether the octave of a note is valid according to
        the conventions of Western music theory.
        If the octave is invalid, a ValueError is raised with a descriptive
        error message.

        Raises:
        -------
            ValueError: If the octave is out of range or has an invalid format.
        """
        if not self.is_valid_octave(self._octave):
            raise ValueError("Invalid octave.")

    def __str__(self) -> str:
        """
        Return a string representation of the object.

        Returns:
        str: The string representation of the object, which is the scientific
        pitch notation of the note.
        """
        return self.spn

    def __eq__(self, other: Union["ScientificPitchNotation", str]) -> bool:
        """Compare two ScientificPitchNotation objects for equality.

        Args:
            other (Union[ScientificPitchNotation, str]): The object to compare
                to. Can be another ScientificPitchNotation object or a string
                in scientific pitch notation format.

        Returns:
            bool: True if the objects are equal, False otherwise.

        Raises:
            TypeError: If other is not a ScientificPitchNotation object or a
            string.
        """
        if isinstance(other, ScientificPitchNotation):
            return self.spn == other.spn
        return self.spn == other



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
# Name:         test_interval.py
# Purpose:      Pymuco Unit Tests
#
# Authors:      German Margon
#
# Copyright:    Copyright © 2022-2023 German Margon. All rights reserved.
# License:      BSD 3-Clause License, see LICENSE
# ------------------------------------------------------------------------------
import unittest

from pymuco import (
    Interval,
    IntervalCalculator,
    IntervalName,
    DEFAULT_OCTAVE,
    C,
    G,
    ScientificPitchNotation
)


class TestIntervalName(unittest.TestCase):
    """
    A subclass of unittest.TestCase that tests the correctness of the
    IntervalName class.

    Attributes
    ----------
    _INTERVAL_NAME: dict
        A dictionary that maps interval names to their values.

    Methods
    -------
    test_interval_name(self):
        Test the correctness of the IntervalName class by comparing its name
        attribute to the keys in the _INTERVAL_NAME dictionary. The function
        iterates through each key-value pair in the _INTERVAL_NAME dictionary
        and asserts that the name attribute of a new IntervalName object
        created with the corresponding value is equal to the key.

        Returns
        -------
        None
    """

    _INTERVAL_NAME = {
        "UNISON": 0,
        "MINOR_SECOND": 1,
        "MAJOR_SECOND": 2,
        "MINOR_THIRD": 3,
        "MAJOR_THIRD": 4,
        "PERFECT_FOURTH": 5,
        "TRITONE": 6,
        "PERFECT_FIFTH": 7,
        "MINOR_SIXTH": 8,
        "MAJOR_SIXTH": 9,
        "MINOR_SEVENTH": 10,
        "MAJOR_SEVENTH": 11,
        "OCTAVE": 12,
        "MINOR_NINTH": 13,
        "MAJOR_NINTH": 14,
        "MINOR_TENTH": 15,
        "MAJOR_TENTH": 16,
        "PERFECT_ELEVENTH": 17,
        "DIMINISHED_TWELFTH": 18,
        "AUGMENTED_ELEVENTH": 19,
        "PERFECT_TWELFTH": 20,
        "MINOR_THIRTEENTH": 21,
        "MAJOR_THIRTEENTH": 22,
        "MINOR_FOURTEENTH": 23,
        "MAJOR_FOURTEENTH": 24,
        "DOUBLE_OCTAVE": 25,
    }

    def test_interval_name(self):
        """
        Test the correctness of the IntervalName class.

        The function iterates through each key-value pair in the
        _INTERVAL_NAME dictionary and asserts that the name attribute of a new
        IntervalName object created with the corresponding value is equal to
        the key.

        Returns
        -------
        None
        """
        for key, value in self._INTERVAL_NAME.items():
            self.assertEqual(IntervalName(value).name, key)


class TestInterval(unittest.TestCase):
    """
    Test case for the Interval class.

    Tests the correctness of its methods get_interval_name(),
    get_interval_direction(), and get_interval_inversion().

    Methods
    -------
    test_get_interval_name()
        Tests the correctness of the get_interval_name() method of the Interval
        class.
    test_get_ascending_interval_direction()
        Tests the correctness of the get_interval_direction() method of the
        Interval class
        on an ascending interval.
    test_get_descending_interval_direction()
        Tests the correctness of the get_interval_direction() method of the
        Interval class
        on a descending interval.
    test_get_interval_inversion()
        Tests the correctness of the get_interval_inversion() method of the
        Interval class.
    """

    _NOTE_MAPPING = {
        "C": 0,
        "B#": 0,
        "D𝄫": 0,
        "C#": 1,
        "Bx": 1,
        "Db": 1,
        "D": 2,
        "Cx": 2,
        "E𝄫": 2,
        "D#": 3,
        "Eb": 3,
        "E": 4,
        "Dx": 4,
        "Fb": 4,
        "F": 5,
        "E#": 5,
        "G𝄫": 5,
        "F#": 6,
        "Ex": 6,
        "Gb": 6,
        "G": 7,
        "A𝄫": 7,
        "Fx": 7,
        "G#": 8,
        "Ab": 8,
        "A": 9,
        "B𝄫": 9,
        "Gx": 9,
        "A#": 10,
        "Bb": 10,
        "B": 11,
        "Ax": 11,
        "Cb": 11,
    }

    def test_get_interval_name(self):
        """
        Test the correctness of the get_interval_name() method of the Interval
        class.

        The function creates two nested for-loops that iterate over all the
        notes in the _NOTE_MAPPING dictionary, creating a pitch object for each
        note at the default octave, and then using these pitch objects to
        create an interval object. The function then calls the
        get_interval_name() method of the Interval class on this interval
        object, which returns an IntervalName object. The function then
        compares the name attribute of this IntervalName object to the interval
        name determined from the _semitones attribute of the interval object
        using the IntervalName class.

        Returns
        -------
        None
        """
        for note1 in self._NOTE_MAPPING:
            for note2 in self._NOTE_MAPPING:
                pitch1 = ScientificPitchNotation(note1, DEFAULT_OCTAVE)
                pitch2 = ScientificPitchNotation(note2, DEFAULT_OCTAVE)
                interval = Interval(pitch1, pitch2)
                self.assertEqual(
                    Interval(pitch1, pitch2).get_interval_name(),
                    IntervalName(interval._semitones).name,
                )

    def test_get_ascending_interval_direction(self):
        """
        Test the correctness of the get_interval_direction() method of the
        Interval class.

        The function creates an interval object with a start pitch of C and an
        end pitch of G, both at the default octave. The function then calls the
        get_interval_direction() method of the Interval class on this interval
        object and compares the output to True, which asserts that the interval
        direction is ascending.

        Returns
        -------
        None
        """
        self.assertTrue(
            Interval(
                ScientificPitchNotation(C, DEFAULT_OCTAVE),
                ScientificPitchNotation(G, DEFAULT_OCTAVE),
            ).get_interval_direction(),
            True,
        )

    def test_get_descending_interval_direction(self):
        """
        Test the correctness of the get_interval_direction() method of the
        Interval class.

        The function creates an interval object with a start pitch of G and an
        end pitch of C, both at the default octave. The function then calls the
        get_interval_direction() method of the Interval class on this interval
        object and compares the output to False, which asserts that the
        interval direction is descending.

        Returns
        -------
        None
        """
        self.assertFalse(
            Interval(
                ScientificPitchNotation(G, DEFAULT_OCTAVE),
                ScientificPitchNotation(C, DEFAULT_OCTAVE),
            ).get_interval_direction(),
            False,
        )

    def test_get_interval_inversion(self):
        """
        This unit test function tests the correctness of the
        get_interval_inversion() method of the Interval class.

        It does this by creating two nested for-loops that iterate over all
        the notes in the _NOTE_MAPPING dictionary, creating a pitch object for
        each note at the default octave, and then using these pitch objects to
        create an interval object.

        The function then calls the get_interval_inversion() method of the
        Interval class on this interval object, which returns an Interval
        object representing the inversion of the original interval.

        The function then compares the name attribute of this new Interval
        object to the interval name determined from the _semitones attribute of
        the original interval object using the IntervalName class.

        Returns:
        None
        """
        for note1 in self._NOTE_MAPPING:
            for note2 in self._NOTE_MAPPING:
                pitch1 = ScientificPitchNotation(note1, DEFAULT_OCTAVE)
                pitch2 = ScientificPitchNotation(note2, DEFAULT_OCTAVE)
                interval = Interval(pitch1, pitch2)
                self.assertEqual(
                    interval.get_interval_inversion(),
                    IntervalName(interval._get_semitones()).name.replace(
                        "_", " "
                    ),
                )


class TestIntervalCalculator(unittest.TestCase):
    """
    Test case for the IntervalCalculator class.

    Methods:
    -------
    test_calculate_ascending_interval():
        A unit test that checks if the calculate_ascending_interval method in
        the IntervalCalculator class is correctly calculating the ascending
        interval between two scientific pitch notations (SPN).
    test_calculate_descending_interval():
        A unit test that checks if the calculate_descending_interval method in
        the IntervalCalculator class is correctly calculating the descending
        interval between two scientific pitch notations (SPN).

    The tests check all possible intervals within an octave for each note in a
    note mapping. The expected outcome is that the calculated intervals have
    the correct number of semitones.
    """

    _NOTE_MAPPING = {
        "C": 0,
        "B#": 0,
        "D𝄫": 0,
        "C#": 1,
        "Bx": 1,
        "Db": 1,
        "D": 2,
        "Cx": 2,
        "E𝄫": 2,
        "D#": 3,
        "Eb": 3,
        "E": 4,
        "Dx": 4,
        "Fb": 4,
        "F": 5,
        "E#": 5,
        "G𝄫": 5,
        "F#": 6,
        "Ex": 6,
        "Gb": 6,
        "G": 7,
        "A𝄫": 7,
        "Fx": 7,
        "G#": 8,
        "Ab": 8,
        "A": 9,
        "B𝄫": 9,
        "Gx": 9,
        "A#": 10,
        "Bb": 10,
        "B": 11,
        "Ax": 11,
        "Cb": 11,
    }

    def test_calculate_ascending_interval(self):
        """
        Test the `calculate_ascending_interval()` method of the
        `IntervalCalculator` class to ensure that it accurately calculates the
        ascending interval between two `ScientificPitchNotation` instances for
        all possible intervals within an octave.

        The method creates an instance of the `ScientificPitchNotation` class
        and an instance of the `IntervalCalculator` class. It then calls the
        `calculate_ascending_interval()` method with an `IntervalName` and the
        `ScientificPitchNotation` instance as arguments to get the calculated
        interval note.

        It creates a new `ScientificPitchNotation` instance for the interval
        note, calculates the direction of the interval using the `Interval`
        class, and adjusts the octave if the interval is descending. It then
        creates an instance of the `Interval` class with the original
        `ScientificPitchNotation` instance and the interval
        `ScientificPitchNotation` instance, calculates the semitones between
        them, and checks if the calculated semitones equal the expected
        semitones using the `assertEqual()` method of the `unittest.TestCase`
        class.

        Parameters:
        -----------
        self: TestInterval
            An instance of the `TestInterval` class.

        Returns:
        --------
        None

        Raises:
        -------
        AssertionError
            If the calculated semitones do not equal the expected semitones for
            any interval.
        """

        for note in self._NOTE_MAPPING:
            spn = ScientificPitchNotation(note, DEFAULT_OCTAVE)
            ic = IntervalCalculator()
            for i in range(0, 12):
                interval_note = ic.calculate_ascending_interval(
                    IntervalName(i), spn
                )
                interval_note_spn = ScientificPitchNotation(
                    interval_note, DEFAULT_OCTAVE
                )
                interval_direction = Interval(
                    spn, interval_note_spn
                ).get_interval_direction()
                if interval_direction is False:
                    interval_note_spn.octave += 1
                interval = Interval(spn, interval_note_spn)
                interval._get_semitones()
                self.assertEqual(interval._get_semitones(), i)

    def test_calculate_descending_interval(self):
        """
        A unit test that checks if the `calculate_descending_interval` method
        in the `IntervalCalculator` class is correctly calculating the
        descending interval between two scientific pitch notations (SPN). The
        test covers all possible intervals within an octave (12 semitones) for
        each note in a note mapping.

        Parameters
        ----------
        None

        Returns
        -------
        None

        Notes
        -----
        This test creates an instance of the `ScientificPitchNotation` class
        and an instance of the `IntervalCalculator` class. It then calls the
        `calculate_descending_interval` method with an `IntervalName` and the
        `ScientificPitchNotation` instance as arguments to get the calculated
        interval note.

        It creates a new `ScientificPitchNotation` instance for the interval
        note and creates an instance of the `Interval` class with the original
        SPN and the interval SPN. It calculates the semitones between them and
        checks if the calculated semitones equal the expected semitones using
        the `assertEqual` method of the `unittest.TestCase` class.

        If the calculated semitones do not equal the expected semitones, the
        test assumes that the interval is ascending rather than descending and
        adjusts the octave of the interval SPN accordingly.

        The purpose of this test is to ensure that the `IntervalCalculator`
        class is accurately calculating descending intervals between SPNs, and
        that the calculated intervals have the correct number of semitones.
        """
        for note in self._NOTE_MAPPING:
            spn = ScientificPitchNotation(note, DEFAULT_OCTAVE)
            ic = IntervalCalculator()
            for i in range(0, 12):
                interval_note = ic.calculate_descending_interval(
                    IntervalName(i), spn
                )
                interval_note_spn = ScientificPitchNotation(
                    interval_note, DEFAULT_OCTAVE
                )
                interval = Interval(spn, interval_note_spn)
                interval._get_semitones()
                if interval._get_semitones() != i:
                    interval_note_spn.octave -= 1
                self.assertEqual(interval._get_semitones(), i)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
# Name:         test_player.py
# Purpose:      Pymuco Unit Tests
#
# Authors:      German Margon
#
# Copyright:    Copyright © 2022-2023 German Margon. All rights reserved.
# License:      BSD 3-Clause License, see LICENSE
# ------------------------------------------------------------------------------
import unittest
from unittest.mock import Mock

from pymuco import (
    Player,
    AudioConverter,
    MusicComputationNotation,
    NoteDuration,
    ScientificPitchNotation
)


class TestPlayer(unittest.TestCase):
    """
    Test the functionality of the Player class.

    This test suite contains test cases that verify the correct behavior of the
    methods of the Player class.

    Methods
    -------
    test_play_with_valid_sample()
        Test the play function with a valid music notation sample.
    test_play_with_invalid_sample()
        Test the play function with an invalid audio sample.

    Notes
    -----
    These test cases ensure that the play function of the Player class
    correctly plays an audio sample generated from a valid music notation and
    raises an appropriate error when an invalid sample is provided.

    Returns
    -------
    None
    """

    def test_play_with_valid_sample(self):
        """
        Test the play function with a valid music notation sample.

        Parameters
        ----------
        None

        Returns
        -------
        None

        Raises
        ------
        AssertionError
            If the audio sample is empty.

        Notes
        -----
        This test case verifies that the play function of the Player class can
        play an audio sample generated by the `convert_to_audio` method of the
        `AudioConverter` class using a valid `MusicComputationNotation` object.

        The test creates a `MusicComputationNotation` object, adds a note with
        a whole note duration to it, and generates an audio sample using an
        `AudioConverter` object. A mock object is used to simulate a valid file
        path for the audio sample. The validity of the file path is checked by
        calling the `_validate_file_path` method of the `AudioConverter`
        object, which should return None.

        The test then checks whether the `_validate_file_path` method has been
        called exactly once. If the validation is successful, the test proceeds
        to play the generated audio sample using the `play` function of the
        `Player` class.

        Finally, the test asserts that the length of the sample list of the
        `AudioConverter` object is greater than 0, which ensures that the audio
        sample has been successfully generated and played.
        """
        notation = MusicComputationNotation()
        C4 = ScientificPitchNotation("C", 4)
        whole_note = NoteDuration("WHOLE_NOTE")
        notation.add_block(C4, whole_note)
        c_sample = AudioConverter()
        c_sample._validate_file_path = Mock(return_value=None)
        c_sample.convert_to_audio(notation)

        c_sample._validate_file_path.assert_called_once()

        player = Player()
        player.play(c_sample)
        assert len(c_sample._sample_list) > 0, "Audio sample is empty"

    def test_play_with_invalid_sample(self):
        """
        Test the play function with an invalid audio sample.

        This test case verifies that the play function of the Player class
        raises a TypeError when an invalid audio sample is passed as an
        argument.

        The test creates a Player object and attempts to play a string object
        that does not represent a valid audio sample. The test uses the
        assertRaises context manager to verify that a TypeError is raised when
        the play function is called with an invalid sample.

        This test ensures that the play function of the Player class correctly
        handles invalid input and raises the appropriate error, helping to
        prevent unexpected behavior or crashes during execution.

        Returns
        -------
        None

        Raises
        ------
        AssertionError
            If the play function does not raise a TypeError when called with an
            invalid sample.
        """
        player = Player()
        with self.assertRaises(TypeError):
            player.play("invalid_sample")



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
# Name:         NoteFrequencyConverter.py
# Purpose:      Pymuco classes
#
# Authors:      German Margon
#
# Copyright:    Copyright © 2022-2023 German Margon. All rights reserved.
# License:      BSD 3-Clause License, see LICENSE
# ------------------------------------------------------------------------------
from .MIDIUtils import MIDINoteConverter
from .ScientificPitchNotation import ScientificPitchNotation


class NoteFrequencyConverter:
    """
    A utility class for converting between Scientific Pitch Notation
    (note names), MIDI note numbers, and frequencies.

    This class provides utility functions to convert between Scientific Pitch
    Notation (note names), MIDI note numbers, and frequencies. The class
    contains two static methods.

    Attributes
    ----------
    None

    Methods
    -------
    get_frequency_from_midi_number(note_number):
        Returns the corresponding frequency in Hz using the formula
        frequency = 440 * 2 ** ((note_number - 69) / 12).

        Parameters
        ----------
        note_number : int
            The MIDI note number to convert.

        Returns
        -------
        float
            The frequency in Hz.

    get_frequency_from_note_name(musical_note):
        Returns the corresponding frequency in Hz from a note name in
        Scientific Pitch Notation.

        Parameters
        ----------
        musical_note : str or ScientificPitchNotation
            The note name to convert. It can be either a string in
            Scientific Pitch Notation or an instance of ScientificPitchNotation
            class.

        Returns
        -------
        float
            The frequency in Hz.

        Raises
        ------
        ValueError
            If the musical_note argument is not a valid instance of
            ScientificPitchNotation class or a valid note name in Scientific
            Pitch Notation.

    Notes
    -----
    The class uses the MIDINoteConverter class from the MIDIUtils module to
    convert the note name to a MIDI note number. The returned frequency is
    rounded to 2 decimal places.
    """

    note_converter = MIDINoteConverter()

    @staticmethod
    def get_frequency_from_midi_number(note_number: int) -> float:
        """
        Given a MIDI note number, returns the corresponding frequency in Hz
        using the formula frequency = 440 * 2 ** ((note_number - 69) / 12)
        This formula is based on the equal temperament tuning system.

        Args:
        -----
        - note_number (int): MIDI note number (integer between 0 and 127)

        Returns:
        --------
        float: frequency in Hz
        """
        return round(440 * 2 ** ((note_number - 69) / 12), 2)

    @staticmethod
    def get_frequency_from_note_name(
        note_name: ScientificPitchNotation,
    ) -> float:
        """
        Given a note name in Scientific Pitch Notation, returns the
        corresponding frequency in Hz.

        Args:
        -----
        - note_name (str): note name in Scientific Pitch Notation
        (e.g. "C4", "A#3")

        Returns:
        --------
        float: frequency in Hz
        """
        if not isinstance(note_name, ScientificPitchNotation):
            raise ValueError(
                "Invalid argument: musical_note must be an instance of "
                "ScientificPitchNotation"
            )

        midi_note_number = (
            NoteFrequencyConverter.note_converter.note_name_to_midi_note(
                note_name
            )
        )
        if midi_note_number is None:
            raise ValueError(
                "Invalid note name. Please provide a valid note name in "
                "Scientific Pitch Notation (e.g. C4, A#3)"
            )

        return round(
            NoteFrequencyConverter.get_frequency_from_midi_number(
                midi_note_number
            ),
            2,
        )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
# Name:         test_circle_of_fifths.py
# Purpose:      Pymuco Unit Tests
#
# Authors:      German Margon
#
# Copyright:    Copyright © 2022-2023 German Margon. All rights reserved.
# License:      BSD 3-Clause License, see LICENSE
# ------------------------------------------------------------------------------
import unittest

from pymuco import CircleOfFifths, ScientificPitchNotation


class TestCircleOfFifths(unittest.TestCase):
    """
    The TestCircleOfFifths class is a unit test that verifies the functionality
    of the CircleOfFifths class.

    The CircleOfFifths class is expected to have two methods, circle_of_fifths
    and relative_minors, which return pre-defined lists of musical keys. The
    test_circle_of_fifths method ensures that the circle_of_fifths method
    returns the expected list of keys, and the test_relative_minors method
    ensures that the relative_minors method returns the expected list of
    relative minor keys. If either method returns a list that is not equal to
    the pre-defined list, the corresponding test will fail.

    Methods
    -------
    test_circle_of_fifths(self):
        Unit test to verify the functionality of the 'circle_of_fifths' method.

    test_relative_minors(self):
        Unit test to verify the functionality of the 'relative_minors' method.
    """

    _CIRCLE_OF_FIFTHS = [
        "C",
        "G",
        "D",
        "A",
        "E",
        "B",
        ["F#", "Gb"],
        "Db",
        "Ab",
        "Eb",
        "Bb",
        "F",
    ]
    _RELATIVE_MINORS = [
        "A",
        "E",
        "B",
        "F#",
        "C#",
        "G#",
        ["D#", "Eb"],
        "Bb",
        "F",
        "C",
        "G",
        "D",
    ]

    def setUp(self):
        """
        Sets up the CircleOfFifths object for use in the test suite.

        This method initializes a CircleOfFifths object and assigns it to the
        instance variable '_circle_of_fifths'. This object can be used in the
        test cases to verify the correctness of the implementation.

        Parameters
        ----------
        self : TestCase
            The test case instance.

        Returns
        -------
        None
        """
        self._circle_of_fifths = CircleOfFifths()

    def test_circle_of_fifths(self):
        """
        Unit test to verify the functionality of the 'circle_of_fifths' method.

        This test ensures that the 'circle_of_fifths' method returns the
        expected circle of fifths object. The test compares the object returned
        by the method with a pre-defined circle of fifths object. If the method
        returns an object that is not equal to the pre-defined object, the test
        will fail.

        Parameters
        ----------
        self : TestCase
            The test case instance.

        Returns
        -------
        None
        """
        self.assertEqual(
            self._circle_of_fifths.circle_of_fifths, self._CIRCLE_OF_FIFTHS
        )

    def test_relative_minors(self):
        """
        Unit test to verify the functionality of the 'circle_of_fifths' method.

        This test ensures that the 'circle_of_fifths' method returns the
        expected circle of fifths object. The test compares the object returned
        by the method with a pre-defined circle of fifths object. If the method
        returns an object that is not equal to the pre-defined object, the test
        will fail.

        Parameters
        ----------
        self : TestCase
            The test case instance.

        Returns
        -------
        None
        """
        self.assertEqual(
            self._circle_of_fifths.relative_minors, self._RELATIVE_MINORS
        )



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
# Name:         KeySignature.py
# Purpose:      Pymuco classes
#
# Authors:      German Margon
#
# Copyright:    Copyright © 2022-2023 German Margon. All rights reserved.
# License:      BSD 3-Clause License, see LICENSE
# ------------------------------------------------------------------------------
from .CircleOfFifths import CircleOfFifths
from .Enharmonic import Enharmonic
from .MusicData import FLAT, MAJOR, MINOR, SHARP, M, m
from .ScientificPitchNotation import ScientificPitchNotation


class KeySignature:
    """
    A class that represents a key signature in music notation.

    Attributes:
    -----------
    _CIRCLE_OF_FIFTHS (list): A list of notes in the circle of fifths.
    _RELATIVE_MINORS (list): A list of relative minor keys for each note in
    the circle of fifths.
    _KEY_SIGNATURE_TYPES (dict): A dictionary that maps key signature types
    (major or minor) to their respective circle of fifths or relative minor
    list.

    Methods:
    --------
    init(self, note: ScientificPitchNotation, mode: str) -> None: Initializes
    a KeySignature object with a given note and mode.

    _get_key_signature_type(self): A private method that returns the key
    signature type (circle of fifths or relative minors) based on the mode of
    the key signature.

    _get_num_of_sharps(self, key_signature_type): A private method that
    returns the number of sharps in a key signature based on the note and key
    signature type.

    _get_num_of_flats(self, key_signature_type): A private method that returns
    the number of flats in a key signature based on the note and key signature
    type.

    get_key_signature(self): Returns the key signature of the object in string
    format.

    get_key_signature_notes(self): Returns a list of notes that have
    accidentals in the key signature.
    """

    _CIRCLE_OF_FIFTHS = CircleOfFifths().circle_of_fifths
    _RELATIVE_MINORS = CircleOfFifths().relative_minors

    _KEY_SIGNATURE_TYPES = {M: _CIRCLE_OF_FIFTHS, m: _RELATIVE_MINORS}

    def __init__(self, note: ScientificPitchNotation, mode: str) -> None:
        """
        The __init__ method initializes a Chord object with a specified note
        in scientific pitch notation and mode. It raises a ValueError if either
        note or mode is not provided.

        Parameters:
        -----------

        note: A ScientificPitchNotation object representing the root note of
        the chord.
        mode: A string representing the mode of the chord, either "major" or
        "minor".

        Returns:
        --------

        None.

        Raises:
        -------

        ValueError: If note or mode is not provided.
        The method sets the self._note attribute to the pitch of the note.
        If the mode is "major", it sets the self._mode attribute to M, and if
        the mode is "minor", it sets the self._mode attribute to m. The method
        also initializes an empty list self._accidental_note to store any
        additional notes that may have an accidental.
        """
        if not note or not mode:
            raise ValueError("Note and mode must be provided")

        self._note = note.pitch
        if mode in MAJOR:
            self._mode = M
        elif mode in MINOR:
            self._mode = m
        self._accidental_note = []

    def _get_key_signature_type(self):
        """
        Returns the type of the key signature for the current mode.

        Returns:
        --------
            A string representing the type of the key signature.

        Note:
        -----
            This method should only be called internally by the KeySignature
            class.
        """
        return KeySignature._KEY_SIGNATURE_TYPES.get(self._mode)

    def _get_num_of_sharps(self, key_signature_type):
        """
        Calculates the number of sharps in the current note's key signature.

        Args:
        -----
            key_signature_type: A string representing the type of the key
            signature.

        Returns:
        --------
            An integer representing the number of sharps in the key signature
            for the current note, or None if the note is not in the given key
            signature.

        Note:
        -----
            This method should only be called internally by the KeySignature
            class.
        """
        for i, note in enumerate(key_signature_type):
            if i <= 5 and self._note == note:
                return i
            elif isinstance(note, list) and i == 6:
                for sub_item in note:
                    if sub_item == self._note and self._note[1] == SHARP:
                        return 6
        return None

    def _get_num_of_flats(self, key_signature_type):
        """
        Calculates the number of flats in the current note's key signature.

        Args:
        -----
            key_signature_type: A string representing the type of the key
            signature.

        Returns:
        --------
            An integer representing the number of flats in the key signature
            for the current note, or None if the note
            is not in the given key signature.

        Note:
        -----
            This method should only be called internally by the KeySignature
            class.
        """
        for i, note in enumerate(key_signature_type):
            if isinstance(note, list) and i == 6:
                for sub_item in note:
                    if sub_item == self._note and self._note[1] == FLAT:
                        return 6
            elif i > 6 and self._note == note:
                return 6 - (i - 6)
        return None

    def get_key_signature(self):
        """
        Calculates the key signature for the current note based on its mode
        and accidental.

        Returns:
        --------
            A string representing the key signature for the current note,
            which could be one of the following formats: '0' (for a key
            signature with no sharps or flats), '1#' to '7#' (for key
            signatures with sharps), or '1b' to '7b' (for key signatures with
            flats). Returns None if the current note's mode is not recognized.

        Note:
        -----
            This method should be called on an instance of the Note class.
        """
        key_signature_type = self._get_key_signature_type()
        if not key_signature_type:
            return None

        num_of_sharps = self._get_num_of_sharps(key_signature_type)
        if num_of_sharps is not None:
            return (
                f"{num_of_sharps}"
                if num_of_sharps == 0
                else f"{num_of_sharps}" + str(SHARP)
            )

        num_of_flats = self._get_num_of_flats(key_signature_type)
        if num_of_flats is not None:
            return (
                f"{num_of_flats}"
                if num_of_flats == 0
                else f"{num_of_flats}" + str(FLAT)
            )

        return None

    def get_key_signature_notes(self):
        """
        Returns a list of notes with accidentals in the current key signature.

        Returns:
        --------
            A list of strings representing the notes with accidentals in the
            current key signature.

        Note:
        -----
            This method should be called on an instance of the Note class.
        """
        key_signature = self.get_key_signature()

        if key_signature is None:
            return None

        if SHARP in key_signature:
            self._accidental_note.insert(
                0, self._CIRCLE_OF_FIFTHS[-1] + str(SHARP)
            )
            for i, note in enumerate(self._CIRCLE_OF_FIFTHS, start=2):
                if i <= int(key_signature[:-1]):
                    self._accidental_note.append(note + str(SHARP))
        elif FLAT in key_signature:
            for index, value in enumerate(reversed(self._CIRCLE_OF_FIFTHS)):
                if index <= int(key_signature[:-1]) and index != 0:
                    if FLAT in value:
                        self._accidental_note.append(value)
                    else:
                        for sub_item in value:
                            if FLAT in sub_item:
                                self._accidental_note.append(sub_item)
                            elif (
                                FLAT not in sub_item and SHARP not in sub_item
                            ):
                                self._accidental_note.append(
                                    Enharmonic(
                                        ScientificPitchNotation(sub_item, 4)
                                    )
                                    .get_enharmonic()
                                    .pitch
                                )
        return self._accidental_note



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
# Name:         test_note_mapping.py
# Purpose:      Pymuco Unit Tests
#
# Authors:      German Margon
#
# Copyright:    Copyright © 2022-2023 German Margon. All rights reserved.
# License:      BSD 3-Clause License, see LICENSE
# ------------------------------------------------------------------------------
import unittest

from pymuco import NoteMapping, ScientificPitchNotation


class TestNoteMapping(unittest.TestCase):
    """
    A test case class for testing the functionality of the NoteMapping class.

    The class has a single test method called test_get_note_mapping, which
    checks if the get_note_mapping method of the NoteMapping class returns the
    expected note mapping dictionary. The _NOTE_MAPPING dictionary is used as a
    reference to compare the returned dictionary with the expected one.

    Attributes:
    -----------
        _NOTE_MAPPING (dict): A dictionary that maps note names to their
        corresponding MIDI numbers.

    Methods:
    --------
        setUp(self): Sets up a NoteMapping instance for testing.
        test_get_note_mapping(self): Tests the get_note_mapping method of the
        NoteMapping class for returning the expected note mapping dictionary.

    Example:
    --------
        >>> test = TestNoteMapping()
        >>> test.setUp()
        >>> test.test_get_note_mapping()
    """

    _NOTE_MAPPING = {
        "C": 0,
        "B#": 0,
        "D𝄫": 0,
        "C#": 1,
        "Bx": 1,
        "Db": 1,
        "D": 2,
        "Cx": 2,
        "E𝄫": 2,
        "D#": 3,
        "Eb": 3,
        "E": 4,
        "Dx": 4,
        "Fb": 4,
        "F": 5,
        "E#": 5,
        "G𝄫": 5,
        "F#": 6,
        "Ex": 6,
        "Gb": 6,
        "G": 7,
        "A𝄫": 7,
        "Fx": 7,
        "G#": 8,
        "Ab": 8,
        "A": 9,
        "B𝄫": 9,
        "Gx": 9,
        "A#": 10,
        "Bb": 10,
        "B": 11,
        "Ax": 11,
        "Cb": 11,
    }

    def setUp(self):
        """
        Method called to prepare the test fixture. It creates an instance of
        the NoteMapping class and assigns it to the
        self.note_mapping attribute.

        Test method:
        ------------
        setUp(self)

        Tested class:
        -------------
        N/A

        Tested method:
        ---------------
        N/A

        Test procedure:
        ---------------
        Create an instance of the NoteMapping class.
        Assign the created instance to the self.note_mapping attribute.

        Expected behavior:
        ------------------
        The self.note_mapping attribute should be an instance of the
        NoteMapping class after calling the setUp method.
        """
        self.note_mapping = NoteMapping()

    def test_get_note_mapping(self):
        """
        Test the `get_note_mapping` method of the `note_mapping` class.

        This method checks if the `get_note_mapping` method of the
        `note_mapping` class returns the expected note mapping dictionary. The
        method is called for each key-value pair in the `_NOTE_MAPPING`
        dictionary, and the returned value is compared with the expected
        `_NOTE_MAPPING` dictionary using the `assertEqual` method.

        Parameters
        ----------
        self: unittest.TestCase
            The instance of the `unittest.TestCase` class.

        Returns
        -------
        None
            This method asserts the correctness of the `get_note_mapping`
            method.

        Raises
        ------
        AssertionError
            If the returned value of the `get_note_mapping` method does not
            match the expected `_NOTE_MAPPING` dictionary.

        See Also
        --------
        note_mapping.get_note_mapping : Returns the note mapping dictionary.

        Example
        -------
        >>> test = TestNoteMapping()
        >>> test.test_get_note_mapping()
        """
        for key, value in self._NOTE_MAPPING.items():
            self.assertEqual(
                self.note_mapping.get_note_mapping(), self._NOTE_MAPPING
            )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
# Name:         test_music_computation_notation.py
# Purpose:      Pymuco Unit Tests
#
# Authors:      German Margon
#
# Copyright:    Copyright © 2022-2023 German Margon. All rights reserved.
# License:      BSD 3-Clause License, see LICENSE
# ------------------------------------------------------------------------------
import unittest

from pymuco import (
    MusicComputationNotation,
    NoteDuration,
    ScientificPitchNotation
)


class TestMusicComputationNotation(unittest.TestCase):
    """
    Test class for MusicComputationNotation.

    Methods
    -------
    test_add_block_one_block:
        Test adding a single block of notes to the notation.
    test_add_single_blocks:
        Test adding multiple single blocks of notes to the notation.
    test_add_multiple_blocks:
        Test adding multiple blocks of notes to the notation.
    test_add_invalid_block:
        Test adding invalid blocks to the notation.
    """

    def setUp(self):
        """
        Sets up the MusicComputationNotation object and pitch and duration
        objects needed for testing.

        Creates the following objects:

        Parameters
        ----------
        self : TestMusicComputationNotation
            Instance of TestMusicComputationNotation class.

        Returns
        -------
        None

        Attributes
        ----------
        music_notation : MusicComputationNotation
            A MusicComputationNotation object for testing.
        C4 : ScientificPitchNotation
            A ScientificPitchNotation object representing the pitch C4.
        E4 : ScientificPitchNotation
            A ScientificPitchNotation object representing the pitch E4.
        G4 : ScientificPitchNotation
            A ScientificPitchNotation object representing the pitch G4.
        half_note : NoteDuration
            A NoteDuration object representing a half note (duration of 0.5
            quarter notes).
        whole_note : NoteDuration
            A NoteDuration object representing a whole note (duration of 4
            quarter notes).
        """
        self.music_notation = MusicComputationNotation()
        self.C4 = ScientificPitchNotation("C", 4)
        self.E4 = ScientificPitchNotation("E", 4)
        self.G4 = ScientificPitchNotation("G", 4)
        self.half_note = NoteDuration(0.5)
        self.whole_note = NoteDuration("WHOLE_NOTE")

    def test_add_block_one_block(self):
        """
        Tests the `add_block` method of the MusicComputationNotation class.

        Parameters:
        -----------
        None

        Setup:
        ------
        music_notation : MusicComputationNotation object
            An instance of MusicComputationNotation.

        C4 : ScientificPitchNotation object
            A ScientificPitchNotation object representing C4.

        whole_note : NoteDuration object
            A NoteDuration object representing a whole note (duration of 4
            quarter notes).

        Actions:
        --------
        Adds a single block consisting of `C4` pitch and `whole_note` duration
        to the `music_notation` object.

        Returns:
        --------
        None

        Assertions:
        -----------
        - Compares the resulting notation to an expected notation.
        - The expected notation is a list containing a single tuple with the
        pitch and duration of the block added.
        - The test checks that the notation stored in the `music_notation`
        object after the addition matches the expected notation.
        """
        self.music_notation.add_block(self.C4, self.whole_note)
        expected_notation = [(self.C4.spn, self.whole_note.duration_length)]
        self.assertEqual(self.music_notation.notation, expected_notation)

    def test_add_single_blocks(self):
        """
        Test that the notation correctly stores single blocks added one at a
        time.

        Parameters:
        -----------
        None

        Setup:
        ------
        Creates a MusicComputationNotation object and adds three blocks of
        notes to it.

        - `music_notation`: a MusicComputationNotation object for testing.
        - `E4`: a ScientificPitchNotation object representing the pitch E4.
        - `G4`: a ScientificPitchNotation object representing the pitch G4.
        - `C4`: a ScientificPitchNotation object representing the pitch C4.
        - `half_note`: a NoteDuration object representing a half note
        (duration of 0.5 quarter notes).
        - `whole_note`: a NoteDuration object representing a whole note
        (duration of 4 quarter notes).

        Expected Result:
        ----------------
        The notation should contain the three blocks in the correct order.

        Test Strategy:
        --------------
        Test the add_block method and the notation property.

        Returns:
        --------
        None
        """
        self.music_notation.add_block(self.E4, self.half_note)
        self.music_notation.add_block(self.G4, self.whole_note)
        self.music_notation.add_block(self.C4, self.half_note)
        expected_notation = [
            (self.E4.spn, self.half_note.duration_length),
            (self.G4.spn, self.whole_note.duration_length),
            (self.C4.spn, self.half_note.duration_length),
        ]
        self.assertEqual(self.music_notation.notation, expected_notation)

    def test_add_multiple_blocks(self):
        """
        Test the functionality of adding multiple blocks to a
        MusicComputationNotation object.

        Parameters:
        -----------
        None

        Setup:
        ------
        music_notation : MusicComputationNotation object
            An instance of MusicComputationNotation to which three blocks of
            notes will be added.

        E4 : Pitch object
            A Pitch object representing the note E4.
        G4 : Pitch object
            A Pitch object representing the note G4.
        C4 : Pitch object
            A Pitch object representing the note C4.

        half_note : Duration object
            A Duration object representing a half note.
        whole_note : Duration object
            A Duration object representing a whole note.

        Returns:
        --------
        None

        Expected Result:
        ----------------
        The notation property of the MusicComputationNotation object should
        contain the three blocks of notes
        in the correct order.

        Test Strategy:
        --------------
        This test case tests the add_block method and the notation property of
        the MusicComputationNotation class.

        """
        self.music_notation.add_block(
            self.E4,
            self.half_note,
            self.G4,
            self.whole_note,
            self.C4,
            self.half_note,
        )
        expected_notation = [
            (self.E4.spn, self.half_note.duration_length),
            (self.G4.spn, self.whole_note.duration_length),
            (self.C4.spn, self.half_note.duration_length),
        ]
        self.assertEqual(self.music_notation.notation, expected_notation)

    def test_add_invalid_block(self):
        """
        Test case for the add_block method of the MusicComputationNotation
        class, when adding invalid blocks.

        Parameters:
        -----------
        None

        Setup:
        ------
        music_notation : MusicComputationNotation object
            An instance of MusicComputationNotation to which blocks with
            invalid arguments will be added.

        C4 : ScientificPitchNotation object
            A ScientificPitchNotation object representing C4.

        whole_note : NoteDuration object
            A NoteDuration object representing a whole note.

        Returns:
        --------
        None

        Expected Result:
        ----------------
        A TypeError should be raised in all three scenarios.

        Test Strategy:
        --------------
        - Call add_block with a string as the pitch argument, and verify that a
        TypeError is raised.
        - Call add_block with a float as the duration argument, and verify that
        a TypeError is raised.
        - Call add_block with an invalid string as the duration argument, and
        verify that a TypeError is raised.
        """
        with self.assertRaises(TypeError):
            self.music_notation.add_block("C4", self.whole_note)
        with self.assertRaises(TypeError):
            self.music_notation.add_block(self.C4, 1.5)
        with self.assertRaises(TypeError):
            self.music_notation.add_block(self.C4, "invalid_duration")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
# Name:         test_scientific_pitch_notation.py
# Purpose:      Pymuco Unit Tests
#
# Authors:      German Margon
#
# Copyright:    Copyright © 2022-2023 German Margon. All rights reserved.
# License:      BSD 3-Clause License, see LICENSE
# ------------------------------------------------------------------------------
import unittest

from pymuco import (
    ACCIDENTALS,
    DEFAULT_OCTAVE,
    NOTE_NAMES,
    C,
    ScientificPitchNotation
)


class TestValidPitch(unittest.TestCase):
    """
    Unit test for the is_valid_pitch and invalid_pitch methods of the
    ScientificPitchNotation class.

    The ScientificPitchNotation class is a class for representing musical
    pitches in scientific pitch notation. This test suite checks if the
    is_valid_pitch and invalid_pitch methods of the ScientificPitchNotation
    class behave as expected.

    Tested class
    ------------
    ScientificPitchNotation

    Test methods
    ------------
    1. test_valid_pitch: checks if all valid pitches are recognized as valid by
    the is_valid_pitch method.
    2. test_invalid_pitch: checks if the invalid_pitch method raises a
    ValueError when an invalid pitch is given as input.

    Test procedure
    ---------------
    1. Create an instance of ScientificPitchNotation.
    2. Call the method being tested with the input.
    3. Verify that the output is correct.

    Returns
    -------
    None
        This function does not return anything.

    See Also
    --------
    is_valid_pitch: Method of ScientificPitchNotation class that checks if a
    pitch is valid. invalid_pitch: Method of ScientificPitchNotation class that
    raises a ValueError if a pitch is invalid.

    Notes
    -----
    In scientific pitch notation, the pitch of a note is represented by a
    letter from A to G, optionally followed by a number indicating the octave
    (e.g., C4 is middle C).

    Examples
    --------
    To run the tests in this class, create an instance of TestValidPitch and
    call the `run` method.

    >>> test_valid_pitch = TestValidPitch()
    >>> test_valid_pitch.run()
    """

    def test_valid_pitch(self):
        """
        Test the `is_valid_pitch` method of the `ScientificPitchNotation`
        class.

        Tests if all valid pitches are recognized as valid by the
        `is_valid_pitch` method of the `ScientificPitchNotation` class. Each
        pitch is created using a combination of note name and accidental from
        the `NOTE_NAMES` and `ACCIDENTALS` constants, and the `DEFAULT_OCTAVE`.
        The `is_valid_pitch` method is called with the pitch, and the result is
        compared with the expected result (True) using the `assertEqual`
        method.

        Parameters:
        -----------
        self : object
            An instance of the `ScientificPitchNotation` test class.

        Returns:
        --------
        None.

        Test method:
        test_valid_pitch(self)

        Tested class:
        `ScientificPitchNotation`

        Tested method:
        `is_valid_pitch`

        Test procedure:
        ---------------
        For each note name and accidental combination in `NOTE_NAMES` and
        `ACCIDENTALS` constants, create a pitch using
        `ScientificPitchNotation`. Call the `is_valid_pitch` method with the
        created pitch and compare the result with the expected result (True)
        using the `assertEqual` method.

        Expected behavior:
        ------------------
        The `is_valid_pitch` method of the `ScientificPitchNotation` class
        should recognize all valid pitches as valid.

        See Also:
        ---------
        `ScientificPitchNotation`
        """
        for note in NOTE_NAMES:
            for accidental in ACCIDENTALS:
                spn = ScientificPitchNotation(
                    note + accidental, DEFAULT_OCTAVE
                )
                result = spn.is_valid_pitch(spn.pitch)
                self.assertEqual(
                    result, True, f"{spn.pitch} should be a valid pitch"
                )

    def test_invalid_pitch(self):
        """
        Test the invalid_pitch method of the ScientificPitchNotation class.

        This test verifies that the invalid_pitch method of the
        ScientificPitchNotation class raises a ValueError when an invalid pitch
        is given as input.

        Parameters
        ----------
        self : unittest.TestCase
            An instance of the unit test case.

        Raises
        ------
        ValueError
            If an invalid pitch is given as input.

        Returns
        -------
        None
            This function does not return anything.

        Test Method
        -----------
        test_invalid_pitch(self)

        Tested Class
        ------------
        ScientificPitchNotation

        Tested Method
        --------------
        invalid_pitch

        Test Procedure
        ---------------
        1. Call the invalid_pitch method of the ScientificPitchNotation class
        with invalid pitches.
        2. Verify that a ValueError is raised.
        3. Compare the raised exception with the expected exception using the
        assertRaises method.

        Expected Behavior
        ------------------
        The invalid_pitch method of the ScientificPitchNotation class should
        raise a ValueError when an invalid pitch is given as input.
        """
        for note in range(ord("H"), ord("Z") + 1):
            with self.assertRaises(ValueError) as context:
                ScientificPitchNotation(str(chr(note)), 4)
            self.assertEqual(str(context.exception), "Invalid pitch.")


class TestValidOctave(unittest.TestCase):
    """
    A unittest class for the ScientificPitchNotation class is_valid_octave
    method.

    Tests:
    - test_valid_octave: checks if all valid octave values ranging from 0 to 10
    are recognized as valid by the is_valid_octave method of the
    ScientificPitchNotation class.
    - test_invalid_negative_octave: checks if the invalid_negative_octave
    method of the ScientificPitchNotation class raises a ValueError when a
    negative octave value is given as input.
    - test_invalid_octave: checks if the invalid_octave method of the
    ScientificPitchNotation class raises a ValueError when an octave value
    outside the range [0, 10] is given as input.

    Methods:
    1. test_valid_octave(self)
    2. test_invalid_negative_octave(self)
    3. test_invalid_octave(self)

    Tested class:
    ScientificPitchNotation

    """

    def test_valid_octave(self):
        """
        Unit test for the is_valid_octave method of the ScientificPitchNotation
        class.

        This test checks if all valid octaves are recognized as valid by the
        is_valid_octave method of the ScientificPitchNotation class. Octaves
        are created by calling the ScientificPitchNotation class with a note
        name (C) and the octave value ranging from 0 to 10. The is_valid_octave
        method is called with the octave value, and the result is compared with
        the expected result (True) using the assertTrue method.

        Test method
        -----------
        test_valid_octave(self)

        Tested class
        ------------
        ScientificPitchNotation

        Tested method
        -------------
        is_valid_octave: Method of ScientificPitchNotation class that checks if
        an octave value is valid.

        Test procedure
        ---------------
        1. Create an instance of ScientificPitchNotation with note name 'C' and
        octave value ranging from 0 to 10.
        2. Call the is_valid_octave method with the octave value.
        3. Verify that the output is True using the assertTrue method.

        Returns
        -------
        None
            This function does not return anything.

        See Also
        --------
        is_valid_octave: Method of ScientificPitchNotation class that checks if
        an octave value is valid.

        Notes
        -----
        In scientific pitch notation, the pitch of a note is represented by a
        letter from A to G, optionally followed by a number indicating the
        octave (e.g., C4 is middle C).

        Examples
        --------
        To run the tests in this method, create an instance of the
        TestValidOctave class and call the `run` method.

        >>> test_valid_octave = TestValidOctave()
        >>> test_valid_octave.run()
        """
        for octave in range(0, 11):
            spn = ScientificPitchNotation(C, octave)
            self.assertTrue(spn.is_valid_octave(spn.octave) == True)

    def test_invalid_negative_octave(self):
        """
        Test the invalid_negative_octave method of the ScientificPitchNotation
        class.

        This method checks if the invalid_negative_octave method of the
        ScientificPitchNotation class raises a ValueError when a negative
        octave value is given as input. The method is called with a negative
        octave, and the raised exception is compared with the expected
        exception using the assertRaises method.

        Parameters
        ----------
        self : unittest.TestCase
            The unittest class.

        Returns
        -------
        None

        Test method:
        test_invalid_negative_octave(self)

        Tested class:
        ScientificPitchNotation

        Tested method:
        invalid_negative_octave

        Test procedure:
        Call the invalid_negative_octave method of the ScientificPitchNotation
        class with a negative octave. Check if a ValueError is raised, and
        compare the raised exception with the expected exception using the
        assertRaises method.

        Expected behavior:
        The invalid_negative_octave method of the ScientificPitchNotation class
        should raise a ValueError when a negative octave value is given as
        input.
        """
        with self.assertRaises(ValueError) as context:
            ScientificPitchNotation(C, -1)
        self.assertEqual(str(context.exception), "Invalid octave.")

    def test_invalid_octave(self):
        """
        Unit test for the invalid_octave method of the ScientificPitchNotation
        class.

        This test checks if the invalid_octave method of the
        ScientificPitchNotation class raises a ValueError when an invalid
        octave is given as input. The method is called with an octave that is
        outside the range [0, 10], and the raised exception is compared with
        the expected exception using the assertRaises method.

        Parameters:
        -----------
        self : unittest.TestCase
            The test case instance

        Returns:
        -------
        None

        Test method:
        test_invalid_octave

        Tested class:
        ScientificPitchNotation

        Tested method:
        invalid_octave

        Test procedure:
        Call the invalid_octave method of the ScientificPitchNotation class
        with an octave value that is outside the range [0, 10]. Check if a
        ValueError is raised, and compare the raised exception with the
        expected exception using the assertRaises method.

        Expected behavior:
        The invalid_octave method of the ScientificPitchNotation class should
        raise a ValueError when an invalid octave is given as input.
        """
        with self.assertRaises(ValueError) as context:
            ScientificPitchNotation(C, 11)
        self.assertEqual(str(context.exception), "Invalid octave.")


# python -m unittest discover .
# python -m unittest discover <directory>

#  ~/code/pymuco/pymuco  python3 -m unittest discover test



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
# Name:         test_chord.py
# Purpose:      Pymuco Unit Tests
#
# Authors:      German Margon
#
# Copyright:    Copyright © 2022-2023 German Margon. All rights reserved.
# License:      BSD 3-Clause License, see LICENSE
# ------------------------------------------------------------------------------
import unittest

from pymuco import Chord, ChordType, ScientificPitchNotation


class TestChordType(unittest.TestCase):
    """
    A class to test the ChordType enum and its associated methods.

    Attributes
    ----------
    _CHORD_TYPE : dict
        A dictionary of chord types with keys as chord names and values as
        lists of intervals.

    Methods
    -------
    test_chord_name():
        Test that the chord names defined in the ChordType enum match their
        corresponding keys in the _CHORD_TYPE dictionary.

    test_chord_sequence():
        Test that the chord types defined in the ChordType enum match their
        corresponding sequences of intervals.
    """

    _CHORD_TYPE = {
        "MAJOR": [4, 7],
        "MINOR": [3, 7],
        "DIMINISHED": [3, 6],
        "AUGMENTED": [4, 8],
        "MAJOR_SEVENTH": [4, 7, 11],
        "MINOR_SEVENTH": [3, 7, 10],
        "DOMINANT_SEVENTH": [4, 7, 10],
        "HALF_DIMINISHED": [3, 6, 10],
        "SUSPENDED_SECOND": [2, 7],
        "SUSPENDED_FOURTH": [5, 7],
    }

    def test_chord_name(self):
        """
        Test that the chord names defined in the ChordType enum match their
        corresponding keys in the _CHORD_TYPE dictionary.

        This test loops over each key in the _CHORD_TYPE dictionary and
        verifies that the name associated with the same key in the ChordType
        enum is equal to the key itself using the assertEqual() method of the
        unittest module.

        Raises:
        -------
            AssertionError: If any of the comparisons fail.

        """
        for key in self._CHORD_TYPE.keys():
            self.assertEqual(ChordType[key].name, key)

    def test_chord_sequence(self):
        """
        Test that the chord types defined in the ChordType enum match their
        corresponding sequences of intervals.

        This test iterates through each key-value pair in the _CHORD_TYPE
        dictionary and checks that the value, which is a sequence of intervals
        representing the chord type, is equal to the tuple of intervals
        associated with the same key in the ChordType enum.

        Raises an AssertionError if any of the comparisons fail.

        Examples
        --------
        >>> chord = ChordType.MAJOR_TRIAD
        >>> chord.value
        (Interval.MAJOR_THIRD, Interval.PERFECT_FIFTH)

        >>> tonality = Tonality(ScientificPitchNotation('C4'),
        TonalityType.MAJOR)
        >>> sequence = [chord, chord, chord, chord, ChordType.DOMINANT_SEVENTH,
        ChordType.MINOR_SEVENTH, chord]
        >>> chord_sequence = tonality.get_chord_sequence(sequence)
        >>> chord_sequence
        [(Pitch(C4), Pitch(E4), Pitch(G4)), (Pitch(C4), Pitch(E4), Pitch(G4)),
        (Pitch(C4), Pitch(E4), Pitch(G4)), (Pitch(C4), Pitch(E4), Pitch(G4)),
        (Pitch(C4), Pitch(E4), Pitch(G4), Pitch(Bf4)), (Pitch(C4), Pitch(Ef4),
        Pitch(G4), Pitch(Bf4)), (Pitch(C4), Pitch(E4), Pitch(G4))]
        """
        for key, value in self._CHORD_TYPE.items():
            self.assertEqual(ChordType[key].value, tuple(value))


class TestChord(unittest.TestCase):
    """
    A class for testing the `Chord` class.
    """

    _MAJOR_CHORD_NOTES = {
        "C_MAJOR": ["C", "E", "G"],
        "D_MAJOR": ["D", "F#", "A"],
        "E_MAJOR": ["E", "G#", "B"],
        "F_MAJOR": ["F", "A", "C"],
        "G_MAJOR": ["G", "B", "D"],
        "A_MAJOR": ["A", "C#", "E"],
        "B_MAJOR": ["B", "D#", "F#"],
        "C#_MAJOR": ["C#", "F", "G#"],
        "D#_MAJOR": ["D#", "G", "A#"],
        "F#_MAJOR": ["F#", "A#", "C#"],
        "G#_MAJOR": ["G#", "C", "D#"],
        "A#_MAJOR": ["A#", "D", "F"],
        "Db_MAJOR": ["Db", "F", "Ab"],
        "Eb_MAJOR": ["Eb", "G", "Bb"],
        "Gb_MAJOR": ["Gb", "Bb", "Db"],
        "Ab_MAJOR": ["Ab", "C", "Eb"],
        "Bb_MAJOR": ["Bb", "D", "F"],
    }

    _MINOR_CHORD_NOTES = {
        "C_MINOR": ["C", "Eb", "G"],
        "D_MINOR": ["D", "F", "A"],
        "E_MINOR": ["E", "G", "B"],
        "F_MINOR": ["F", "Ab", "C"],
        "G_MINOR": ["G", "Bb", "D"],
        "A_MINOR": ["A", "C", "E"],
        "B_MINOR": ["B", "D", "F#"],
        "C#_MINOR": ["C#", "E", "G#"],
        "D#_MINOR": ["D#", "F#", "A#"],
        "F#_MINOR": ["F#", "A", "C#"],
        "G#_MINOR": ["G#", "B", "D#"],
        "A#_MINOR": ["A#", "C#", "F"],
        "Cb_MINOR": ["Cb", "D", "Gb"],
        "Db_MINOR": ["Db", "E", "Ab"],
        "Eb_MINOR": ["Eb", "Gb", "Bb"],
        "Gb_MINOR": ["Gb", "A", "Db"],
        "Ab_MINOR": ["Ab", "B", "Eb"],
        "Bb_MINOR": ["Bb", "Db", "F"],
    }

    _DIMINISHED_CHORD_NOTES = {
        "C_DIMINISHED": ["C", "Eb", "F#"],
        "D_DIMINISHED": ["D", "F", "G#"],
        "E_DIMINISHED": ["E", "G", "A#"],
        "F_DIMINISHED": ["F", "Ab", "B"],
        "G_DIMINISHED": ["G", "Bb", "C#"],
        "A_DIMINISHED": ["A", "C", "D#"],
        "B_DIMINISHED": ["B", "D", "F"],
        "C#_DIMINISHED": ["C#", "E", "G"],
        "D#_DIMINISHED": ["D#", "F#", "A"],
        "F#_DIMINISHED": ["F#", "A", "C"],
        "G#_DIMINISHED": ["G#", "B", "D"],
        "A#_DIMINISHED": ["A#", "C#", "E"],
        "Cb_DIMINISHED": ["Cb", "D", "F"],
        "Db_DIMINISHED": ["Db", "E", "G"],
        "Eb_DIMINISHED": ["Eb", "Gb", "A"],
        "Gb_DIMINISHED": ["Gb", "A", "C"],
        "Ab_DIMINISHED": ["Ab", "B", "D"],
        "Bb_DIMINISHED": ["Bb", "Db", "E"],
    }

    _AUGMENTED_CHORD_NOTES = {
        "C_AUGMENTED": ["C", "E", "G#"],
        "D_AUGMENTED": ["D", "F#", "A#"],
        "E_AUGMENTED": ["E", "G#", "C"],
        "F_AUGMENTED": ["F", "A", "C#"],
        "G_AUGMENTED": ["G", "B", "D#"],
        "A_AUGMENTED": ["A", "C#", "F"],
        "B_AUGMENTED": ["B", "D#", "G"],
        "C#_AUGMENTED": ["C#", "F", "A"],
        "D#_AUGMENTED": ["D#", "G", "B"],
        "F#_AUGMENTED": ["F#", "A#", "D"],
        "G#_AUGMENTED": ["G#", "C", "E"],
        "A#_AUGMENTED": ["A#", "D", "F#"],
        "Cb_AUGMENTED": ["Cb", "Eb", "G"],
        "Db_AUGMENTED": ["Db", "F", "A"],
        "Eb_AUGMENTED": ["Eb", "G", "B"],
        "Gb_AUGMENTED": ["Gb", "Bb", "D"],
        "Ab_AUGMENTED": ["Ab", "C", "E"],
        "Bb_AUGMENTED": ["Bb", "D", "Gb"],
    }

    _MAJOR_SEVENTH_CHORD_NOTES = {
        "C_MAJOR_SEVENTH": ["C", "E", "G", "B"],
        "D_MAJOR_SEVENTH": ["D", "F#", "A", "C#"],
        "E_MAJOR_SEVENTH": ["E", "G#", "B", "D#"],
        "F_MAJOR_SEVENTH": ["F", "A", "C", "E"],
        "G_MAJOR_SEVENTH": ["G", "B", "D", "F#"],
        "A_MAJOR_SEVENTH": ["A", "C#", "E", "G#"],
        "B_MAJOR_SEVENTH": ["B", "D#", "F#", "A#"],
        "C#_MAJOR_SEVENTH": ["C#", "F", "G#", "C"],
        "D#_MAJOR_SEVENTH": ["D#", "G", "A#", "D"],
        "F#_MAJOR_SEVENTH": ["F#", "A#", "C#", "F"],
        "G#_MAJOR_SEVENTH": ["G#", "C", "D#", "G"],
        "A#_MAJOR_SEVENTH": ["A#", "D", "F", "A"],
        "Cb_MAJOR_SEVENTH": ["Cb", "Eb", "Gb", "Bb"],
        "Db_MAJOR_SEVENTH": ["Db", "F", "Ab", "C"],
        "Eb_MAJOR_SEVENTH": ["Eb", "G", "Bb", "D"],
        "Gb_MAJOR_SEVENTH": ["Gb", "Bb", "Db", "F"],
        "Ab_MAJOR_SEVENTH": ["Ab", "C", "Eb", "G"],
        "Bb_MAJOR_SEVENTH": ["Bb", "D", "F", "A"],
    }

    _MINOR_SEVENTH_CHORD_NOTES = {
        "C_MAJOR_SEVENTH": ["C", "Eb", "G", "Bb"],
        "D_MINOR_SEVENTH": ["D", "F", "A", "C"],
        "E_MINOR_SEVENTH": ["E", "G", "B", "D"],
        "F_MINOR_SEVENTH": ["F", "Ab", "C", "Eb"],
        "G_MINOR_SEVENTH": ["G", "Bb", "D", "F"],
        "A_MINOR_SEVENTH": ["A", "C", "E", "G"],
        "B_MINOR_SEVENTH": ["B", "D", "F#", "A"],
        "C#_MINOR_SEVENTH": ["C#", "E", "G#", "B"],
        "D#_MINOR_SEVENTH": ["D#", "F#", "A#", "C#"],
        "F#_MINOR_SEVENTH": ["F#", "A", "C#", "E"],
        "G#_MINOR_SEVENTH": ["G#", "B", "D#", "F#"],
        "A#_MINOR_SEVENTH": ["A#", "C#", "F", "G#"],
        "Cb_MINOR_SEVENTH": ["Cb", "D", "Gb", "A"],
        "Db_MINOR_SEVENTH": ["Db", "E", "Ab", "B"],
        "Eb_MINOR_SEVENTH": ["Eb", "Gb", "Bb", "Db"],
        "Gb_MINOR_SEVENTH": ["Gb", "A", "Db", "E"],
        "Ab_MINOR_SEVENTH": ["Ab", "B", "Eb", "Gb"],
        "Bb_MINOR_SEVENTH": ["Bb", "Db", "F", "Ab"],
    }

    _DOMINANT_SEVENTH_CHORD_NOTES = {
        "C_DOMINANT_SEVENTH": ["C", "E", "G", "A#"],
        "D_DOMINANT_SEVENTH": ["D", "F#", "A", "C"],
        "E_DOMINANT_SEVENTH": ["E", "G#", "B", "D"],
        "F_DOMINANT_SEVENTH": ["F", "A", "C", "D#"],
        "G_DOMINANT_SEVENTH": ["G", "B", "D", "F"],
        "A_DOMINANT_SEVENTH": ["A", "C#", "E", "G"],
        "B_DOMINANT_SEVENTH": ["B", "D#", "F#", "A"],
        "C#_DOMINANT_SEVENTH": ["C#", "F", "G#", "B"],
        "D#_DOMINANT_SEVENTH": ["D#", "G", "A#", "C#"],
        "F#_DOMINANT_SEVENTH": ["F#", "A#", "C#", "E"],
        "G#_DOMINANT_SEVENTH": ["G#", "C", "D#", "F#"],
        "A#_DOMINANT_SEVENTH": ["A#", "D", "F", "G#"],
        "Cb_DOMINANT_SEVENTH": ["Cb", "Eb", "Gb", "A"],
        "Db_DOMINANT_SEVENTH": ["Db", "F", "Ab", "B"],
        "Eb_DOMINANT_SEVENTH": ["Eb", "G", "Bb", "Db"],
        "Gb_DOMINANT_SEVENTH": ["Gb", "Bb", "Db", "E"],
        "Ab_DOMINANT_SEVENTH": ["Ab", "C", "Eb", "Gb"],
        "Bb_DOMINANT_SEVENTH": ["Bb", "D", "F", "Ab"],
    }

    _HALF_DIMINISHED_CHORD_NOTES = {
        "C_HALF_DIMINISHED": ["C", "Eb", "F#", "Bb"],
        "D_HALF_DIMINISHED": ["D", "F", "G#", "C"],
        "E_HALF_DIMINISHED": ["E", "G", "A#", "D"],
        "F_HALF_DIMINISHED": ["F", "Ab", "B", "Eb"],
        "G_HALF_DIMINISHED": ["G", "Bb", "C#", "F"],
        "A_HALF_DIMINISHED": ["A", "C", "D#", "G"],
        "B_HALF_DIMINISHED": ["B", "D", "F", "A"],
        "C#_HALF_DIMINISHED": ["C#", "E", "G", "B"],
        "D#_HALF_DIMINISHED": ["D#", "F#", "A", "C#"],
        "F#_HALF_DIMINISHED": ["F#", "A", "C", "E"],
        "G#_HALF_DIMINISHED": ["G#", "B", "D", "F#"],
        "A#_HALF_DIMINISHED": ["A#", "C#", "E", "G#"],
        "Cb_HALF_DIMINISHED": ["Cb", "D", "F", "A"],
        "Db_HALF_DIMINISHED": ["Db", "E", "G", "B"],
        "Eb_HALF_DIMINISHED": ["Eb", "Gb", "A", "Db"],
        "Gb_HALF_DIMINISHED": ["Gb", "A", "C", "E"],
        "Ab_HALF_DIMINISHED": ["Ab", "B", "D", "Gb"],
        "Bb_HALF_DIMINISHED": ["Bb", "Db", "E", "Ab"],
    }

    _SUSPENDED_SECOND_CHORD_NOTES = {
        "C_SUSPENDED_SECOND": ["C", "E", "G#"],
        "D_SUSPENDED_SECOND": ["D", "F#", "A#"],
        "E_SUSPENDED_SECOND": ["E", "G#", "C"],
        "F_SUSPENDED_SECOND": ["F", "A", "C#"],
        "G_SUSPENDED_SECOND": ["G", "B", "D#"],
        "A_SUSPENDED_SECOND": ["A", "C#", "F"],
        "B_SUSPENDED_SECOND": ["B", "D#", "G"],
        "C#_SUSPENDED_SECOND": ["C#", "F", "A"],
        "D#_SUSPENDED_SECOND": ["D#", "G", "B"],
        "F#_SUSPENDED_SECOND": ["F#", "A#", "D"],
        "G#_SUSPENDED_SECOND": ["G#", "C", "E"],
        "A#_SUSPENDED_SECOND": ["A#", "D", "F#"],
        "Cb_SUSPENDED_SECOND": ["Cb", "Eb", "G"],
        "Db_SUSPENDED_SECOND": ["Db", "F", "A"],
        "Eb_SUSPENDED_SECOND": ["Eb", "G", "B"],
        "Gb_SUSPENDED_SECOND": ["Gb", "Bb", "D"],
        "Ab_SUSPENDED_SECOND": ["Ab", "C", "E"],
        "Bb_SUSPENDED_SECOND": ["Bb", "C", "F"],
    }

    _SUSPENDED_FOURTH_CHORD_NOTES = {
        "C_SUSPENDED_FOURTH": ["C", "F", "G"],
        "D_SUSPENDED_FOURTH": ["D", "G", "A"],
        "E_SUSPENDED_FOURTH": ["E", "A", "B"],
        "F_SUSPENDED_FOURTH": ["F", "A#", "C"],
        "G_SUSPENDED_FOURTH": ["G", "C", "D"],
        "A_SUSPENDED_FOURTH": ["A", "D", "E"],
        "B_SUSPENDED_FOURTH": ["B", "E", "F#"],
        "C#_SUSPENDED_FOURTH": ["C#", "F#", "G#"],
        "D#_SUSPENDED_FOURTH": ["D#", "G#", "A#"],
        "F#_SUSPENDED_FOURTH": ["F#", "B", "C#"],
        "G#_SUSPENDED_FOURTH": ["G#", "C#", "D#"],
        "A#_SUSPENDED_FOURTH": ["A#", "D#", "F"],
        "Cb_SUSPENDED_FOURTH": ["Cb", "E", "Gb"],
        "Db_SUSPENDED_FOURTH": ["Db", "Gb", "Ab"],
        "Eb_SUSPENDED_FOURTH": ["Eb", "Ab", "Bb"],
        "Gb_SUSPENDED_FOURTH": ["Gb", "B", "Db"],
        "Ab_SUSPENDED_FOURTH": ["Ab", "Db", "Eb"],
        "Bb_SUSPENDED_FOURTH": ["Bb", "Eb", "F"],
    }

    def test_get_major_chord_notes(self):
        """
        Test that the `get_chord_notes()` method returns the correct notes for
        major chords.

        This test iterates through each key-value pair in the
        `_MAJOR_CHORD_NOTES` dictionary, where the keys are in the format
        "<root-note>_MAJOR" in scientific pitch notation, and the values are
        the expected notes in the chord. For each pair, a major chord is
        created with the root note specified by the key using the `Chord` class
        and the `MAJOR` chord type. The `get_chord_notes()` method is then
        called on this chord, and the resulting notes are compared with the
        expected value in the dictionary.

        Raises:
        -------
            AssertionError: If any of the comparisons fail.
        """
        for key, value in self._MAJOR_CHORD_NOTES.items():
            chord_notes = Chord(
                ScientificPitchNotation(key.split("_")[0], 4), ChordType.MAJOR
            ).get_chord_notes()
        self.assertEqual(chord_notes, value)

    def test_get_minor_chord_notes(self):
        """
        Test that the get_chord_notes() method returns the correct notes for
        minor chords.

        This test loops over each key-value pair in the _MINOR_CHORD_NOTES
        dictionary and creates a minor chord with the root note specified by
        the key (in Scientific Pitch Notation), and checks that the notes
        returned by the get_chord_notes() method match the value in the
        dictionary.

        Raises:
        -------
        AssertionError: If any of the comparisons fail.

        Examples
        --------
        >>> chord = Chord(ScientificPitchNotation('C4'), ChordType.MINOR_TRIAD)
        >>> chord.get_chord_notes()
        [Pitch(C4), Pitch(Ef4), Pitch(G4)]

        """
        for key, value in self._MINOR_CHORD_NOTES.items():
            chord_notes = Chord(
                ScientificPitchNotation(key.split("_")[0], 4), ChordType.MINOR
            ).get_chord_notes()
        self.assertEqual(chord_notes, value)

    def test_get_disminished_chord_notes(self):
        """
        Test that the `get_chord_notes()` method returns the correct notes for
        diminished chords.

        This test iterates over each key-value pair in the
        `_DIMINISHED_CHORD_NOTES` dictionary and creates a diminished chord
        with the root note specified by the key (in scientific pitch notation).
        It then checks that the notes returned by the `get_chord_notes()`
        method match the value in the dictionary using the `assertEqual()`
        method from the unittest module.

        Raises:
        -------
            AssertionError: If any of the comparisons fail.
        """
        for key, value in self._DIMINISHED_CHORD_NOTES.items():
            chord_notes = Chord(
                ScientificPitchNotation(key.split("_")[0], 4),
                ChordType.DIMINISHED,
            ).get_chord_notes()
        self.assertEqual(chord_notes, value)

    def test_get_augmented_chord_notes(self):
        """
        Test that the get_chord_notes() method returns the correct notes for
        augmented chords.

        This method iterates over each key-value pair in the
        _AUGMENTED_CHORD_NOTES dictionary, creates an augmented chord with the
        root note specified by the key (in scientific pitch notation), and
        verifies that the notes returned by the get_chord_notes() method match
        the value in the dictionary.

        Raises:
        AssertionError: If any of the comparisons fail.

        """
        for key, value in self._AUGMENTED_CHORD_NOTES.items():
            chord_notes = Chord(
                ScientificPitchNotation(key.split("_")[0], 4),
                ChordType.AUGMENTED,
            ).get_chord_notes()
        self.assertEqual(chord_notes, value)

    def test_get_major_seventh_chord_notes(self):
        """
        Test that the get_chord_notes() method returns the correct notes for
        major seventh chords.

        This method iterates over each key-value pair in the
        _MAJOR_SEVENTH_CHORD_NOTES dictionary, creates a major seventh chord
        with the root note specified by the key (in scientific pitch notation),
        and verifies that the notes returned by the get_chord_notes() method
        match the value in the dictionary.

        Raises:
        AssertionError: If any of the comparisons fail.

        """
        for key, value in self._MAJOR_SEVENTH_CHORD_NOTES.items():
            chord_notes = Chord(
                ScientificPitchNotation(key.split("_")[0], 4),
                ChordType.MAJOR_SEVENTH,
            ).get_chord_notes()
        self.assertEqual(chord_notes, value)

    def test_get_minor_seventh_chord_notes(self):
        """
        Test that the get_chord_notes() method returns the correct notes for
        minor seventh chords.

        This method iterates over each key-value pair in the
        _MINOR_SEVENTH_CHORD_NOTES dictionary, creates a minor seventh chord
        with the root note specified by the key (in scientific pitch notation),
        and verifies that the notes returned by the get_chord_notes() method
        match the value in the dictionary.

        Raises:
        AssertionError: If any of the comparisons fail.

        """
        for key, value in self._MINOR_SEVENTH_CHORD_NOTES.items():
            chord_notes = Chord(
                ScientificPitchNotation(key.split("_")[0], 4),
                ChordType.MINOR_SEVENTH,
            ).get_chord_notes()
        self.assertEqual(chord_notes, value)

    def test_get_dominant_seventh_chord_notes(self):
        """
        Test that the get_chord_notes() method returns the correct notes for
        dominant seventh chords.

        This method iterates over each key-value pair in the
        _DOMINANT_SEVENTH_CHORD_NOTES dictionary, creates a dominant seventh
        chord with the root note specified by the key (in scientific pitch
        notation), and verifies that the notes returned by the
        get_chord_notes() method match the value in the dictionary.

        Raises:
        AssertionError: If any of the comparisons fail.

        """
        for key, value in self._DOMINANT_SEVENTH_CHORD_NOTES.items():
            chord_notes = Chord(
                ScientificPitchNotation(key.split("_")[0], 4),
                ChordType.DOMINANT_SEVENTH,
            ).get_chord_notes()
        self.assertEqual(chord_notes, value)

    def test_get_half_diminished_chord_notes(self):
        """
        Test the functionality of `get_chord_notes()` method for
        half-diminished chords.

        This test loops over each key-value pair in the
        `_HALF_DIMINISHED_CHORD_NOTES` dictionary, creates a half-diminished
        chord with the root note specified by the key (in scientific pitch
        notation) and asserts that the notes returned by the
        `get_chord_notes()` method match the expected value.

        Raises:
        -------
        AssertionError:
            If any of the comparisons fail.

        """
        for key, value in self._HALF_DIMINISHED_CHORD_NOTES.items():
            chord_notes = Chord(
                ScientificPitchNotation(key.split("_")[0], 4),
                ChordType.HALF_DIMINISHED,
            ).get_chord_notes()
        self.assertEqual(chord_notes, value)

    def test_get_suspended_second_chord_notes(self):
        """
        Test the functionality of `get_chord_notes()` method for suspended
        second chords.

        This test loops over each key-value pair in the
        `_SUSPENDED_SECOND_CHORD_NOTES` dictionary, creates a suspended second
        chord with the root note specified by the key (in scientific pitch
        notation) and asserts that the notes returned by the
        `get_chord_notes()` method match the expected value.

        Raises:
        -------
        AssertionError:
            If any of the comparisons fail.

        """
        for key, value in self._SUSPENDED_SECOND_CHORD_NOTES.items():
            chord_notes = Chord(
                ScientificPitchNotation(key.split("_")[0], 4),
                ChordType.SUSPENDED_SECOND,
            ).get_chord_notes()
        self.assertEqual(chord_notes, value)

    def test_get_suspended_fourth_chord_notes(self):
        """
        Test the functionality of `get_chord_notes()` method for suspended
        fourth chords.

        This test loops over each key-value pair in the
        `_SUSPENDED_FOURTH_CHORD_NOTES` dictionary,
        creates a suspended fourth chord with the root note specified by the
        key (in scientific pitch notation) and asserts that the notes returned
        by the `get_chord_notes()` method match the expected value.

        Raises:
        -------
        AssertionError:
            If any of the comparisons fail.
        """
        for key, value in self._SUSPENDED_FOURTH_CHORD_NOTES.items():
            chord_notes = Chord(
                ScientificPitchNotation(key.split("_")[0], 4),
                ChordType.SUSPENDED_FOURTH,
            ).get_chord_notes()
        self.assertEqual(chord_notes, value)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
# Name:         __init__.py
# Purpose:      Pymuco Package
#
# Authors:      German Margon
#
# Copyright:    Copyright © 2022-2023 German Margon. All rights reserved.
# License:      BSD 3-Clause License, see LICENSE
# ------------------------------------------------------------------------------
from .AudioConverter import AudioConverter
from .Chord import Chord, ChordType
from .CircleOfFifths import CircleOfFifths
from .Enharmonic import Enharmonic
from .EnharmonicMapping import EnharmonicMapping
from .Interval import Interval, IntervalCalculator, IntervalName
from .KeySignature import KeySignature
from .MIDIUtils import MIDINoteConverter
from .MidiGenerator import MidiGenerator
from .MusicComputationNotation import MusicComputationNotation
from .MusicData import (
    ACCIDENTALS,
    DEFAULT_OCTAVE,
    FLAT,
    MAJOR,
    MINOR,
    M,
    m,
    NOTE_NAMES,
    SHARP,
    C,
    G
)
from .NoteDuration import NoteDuration, NoteDurationMapping
from .NoteFrequencyConverter import NoteFrequencyConverter
from .NoteMapping import NoteMapping
from .Player import Player
from .Scale import Scale, ScaleData
from .ScientificPitchNotation import ScientificPitchNotation
from .Tonality import Tonality

__version__ = "1.1.5"

__all__ = [
    "AudioConverter",
    "Chord",
    "ChordType",
    "CircleOfFifths",
    "Enharmonic",
    "EnharmonicMapping",
    "Interval",
    "IntervalCalculator",
    "IntervalName",
    "KeySignature",
    "MIDINoteConverter",
    "MidiGenerator",
    "MusicComputationNotation",
    "NoteDuration",
    "NoteDurationMapping",
    "NoteFrequencyConverter",
    "NoteMapping",
    "Player",
    "Scale",
    "ScaleData",
    "ScientificPitchNotation",
    "Tonality",
    # Constants
    "ACCIDENTALS",
    "DEFAULT_OCTAVE",
    "FLAT",
    "MAJOR",
    "MINOR",
    "M",
    "m",
    "NOTE_NAMES",
    "SHARP",
    "C",
    "G"
] 